** FILE:
**    SWAUTIL.FOR - part of program SWAP
** FILE IDENTIFICATION:
**    $Id: SWAUTIL.FOR 2.3.1.10 2002/09/02 08:50:55Z Kroes Exp $
** DESCRIPTION:
**    This file contains program units of SWAP. The program units 
**    in this file utilities and are sorted alphabetically
**    with the first letter of the program unit ranging from A - O.
**    The following program units are in this file:
**        AFGEN, ANALIN, DATONR, FROMTO, 
**        LIMITR8, NRTODA, SHIFTL, SHIFTR, STEPNR
************************************************************************
C ----------------------------------------------------------------------
      REAL*8 FUNCTION AFGEN(TABLE,ILTAB,X)
C ----------------------------------------------------------------------
C     Source             : Kees Rappoldt, 1/86
C     Purpose            : Linear interpolation in table TABLE with
C                          length ILTAB for a given value of the
C                          independent variable X.
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER I,ILTAB
      REAL*8  TABLE(ILTAB),SLOPE,X
C ----------------------------------------------------------------------
      IF (TABLE(1).GE.X)  GOTO 40
      DO 10 I = 3,ILTAB-1,2
        IF (TABLE(I).GE.X) GOTO 30
        IF (TABLE(I).LT.TABLE(I-2)) GOTO 20
 10   CONTINUE
C --- table fully filled, argument larger then last X in table
      AFGEN = TABLE(ILTAB)
      RETURN
C --- table partly filled, argument larger then last X in table
 20   AFGEN = TABLE(I-1)
      RETURN
C --- argument between first and last X in table, interpolation
 30   SLOPE = (TABLE(I+1)-TABLE(I-1))/(TABLE(I)-TABLE(I-2))
      AFGEN = TABLE(I-1) + (X-TABLE(I-2))*SLOPE
      RETURN
C --- argument less or equal to first X in table
 40   AFGEN = TABLE(2)
      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE ANALIN (LINE,BLANK,COMMENT)
C ----------------------------------------------------------------------
C     Date               : 27/10/1997
C     Purpose            : check if LINE is blank or COMMENT line
C                          COMMENT lines start with '*' or '!'
C     Subroutines called :
C     Functions called   :
C ----------------------------------------------------------------------
      IMPLICIT NONE

      CHARACTER LINE*80
      LOGICAL BLANK,COMMENT
C ----------------------------------------------------------------------
C --- local
      INTEGER I
C ----------------------------------------------------------------------
      BLANK = .TRUE.                     
      I = 1
  30  IF (LINE(I:I) .NE. ' ') THEN
        BLANK = .FALSE.
      ELSE IF (I .LT. 80) THEN
        I = I+1
        GOTO 30
      ENDIF

      COMMENT = .FALSE.
      I = 1
40    IF (LINE(I:I).NE.' ') THEN
        IF (LINE(I:I).EQ.'*'.OR.LINE(I:I).EQ.'!') COMMENT = .TRUE.
      ELSEIF (I.LT.80) THEN
        I = I+1
        GOTO 40
      ENDIF

      RETURN 
      END

C ----------------------------------------------------------------------
      SUBROUTINE DATONR (YEAR,MONTH,DAY,DAYNR)
C ----------------------------------------------------------------------
C     Date               : 11/91
C     Purpose            : Conversion of date (YEAR,MONTH,DAY) to    
C                          daynumber (DAYNR)
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE 
      
      INTEGER ILEAP,YEAR,MONTH,DAY,DAYNR
      INTEGER FDM365(12),FDM366(12)

      DATA FDM365/1,32,60,91,121,152,182,213,244,274,305,335/
      DATA FDM366/1,32,61,92,122,153,183,214,245,275,306,336/
C ----------------------------------------------------------------------

C --- is YEAR a leap-year?
      ILEAP=0
      IF (((MOD(YEAR,4).EQ.0).AND.(MOD(YEAR,100).NE.0)).OR.
     $     (MOD(YEAR,400).EQ.0)) ILEAP=1   

C --- calculate daynumber
      IF (ILEAP.EQ.0) THEN
        DAYNR=FDM365(MONTH)+DAY-1
      ELSEIF (ILEAP.EQ.1) THEN
        DAYNR=FDM366(MONTH)+DAY-1
      ENDIF

      RETURN
      END
C ----------------------------------------------------------------------
      INTEGER FUNCTION FROMTO (DAY1,YEAR1,DAY2,YEAR2)
C ----------------------------------------------------------------------
C     Date               : 2/92
C     Purpose            : Calculates number of days between DAY1, YEAR1
C                          and DAY2, YEAR2
C     Subroutines called : DATONR
C     Functions called   : -
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER I,J,DAY1,YEAR1,DAY2,YEAR2,DAYS
C ----------------------------------------------------------------------
      IF (YEAR1.EQ.YEAR2) THEN
        FROMTO = DAY2 - DAY1
      ELSEIF (YEAR2.GT.YEAR1) THEN 
        DAYS=0
        DO 10 I = YEAR1,YEAR2-1
          CALL DATONR (I,12,31,J)
          DAYS = DAYS + J
10      CONTINUE
        FROMTO = DAYS+DAY2-DAY1
      ENDIF

      RETURN
      END

C ----------------------------------------------------------------------
      REAL*8 FUNCTION LIMITR8(P1,P2,X)
C ----------------------------------------------------------------------
C --- Limits X to values within the range between lower limit P1 and
C --- upper limit P2
C ----------------------------------------------------------------------
      IMPLICIT NONE

      REAL*8 P1,P2,X
C ----------------------------------------------------------------------
      LIMITR8=X
      IF (X.GT.P2) LIMITR8=P2
      IF (X.LT.P1) LIMITR8=P1
      RETURN
      END

C ----------------------------------------------------------------------
      SUBROUTINE NRTODA (YEAR,DAYNR,MONTH,DAY)
C ----------------------------------------------------------------------
C     Date               : 11/91
C     Purpose            : Conversion of daynumber (YEAR,DAYNR) to    
C                          date (MONTH,DAY)
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER ILEAP,YEAR,MONTH,DAY,DAYNR
      INTEGER FDM365(12),FDM366(12)

      DATA FDM365/1,32,60,91,121,152,182,213,244,274,305,335/
      DATA FDM366/1,32,61,92,122,153,183,214,245,275,306,336/
C ----------------------------------------------------------------------

C --- is YEAR a leap-year?
      ILEAP=0
      IF (((MOD(YEAR,4).EQ.0).AND.(MOD(YEAR,100).NE.0)).OR.
     $     (MOD(YEAR,400).EQ.0)) ILEAP=1   

C --- calculate month, day
      IF (ILEAP.EQ.0) THEN
        MONTH=12
10      IF (DAYNR.LT.FDM365(MONTH)) THEN
          MONTH=MONTH-1
          GOTO 10
        ENDIF
        DAY=DAYNR-FDM365(MONTH)+1
      ELSEIF (ILEAP.EQ.1) THEN
        MONTH=12
20      IF (DAYNR.LT.FDM366(MONTH)) THEN
          MONTH=MONTH-1
          GOTO 20
        ENDIF
        DAY=DAYNR-FDM366(MONTH)+1
      ENDIF

      RETURN
      END
C-----------------------------------------------------------------------
      SUBROUTINE SHIFTL(STRING)
C-----------------------------------------------------------------------
C     Date               : 26/10/95
C     Purpose            : Shift STRING to the left as far as possible
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER   I,LS,empty

      CHARACTER *(*) STRING
C-----------------------------------------------------------------------
      ls  = len (string)

      empty = 0
      i     = 1
30    if (string(i:i).eq.' ') then
        empty = empty+1
        i     = i+1
        if (i.le.ls) goto 30
      endif

      if (empty.gt.0) then
        do 40 i = 1,ls-empty
          string(i:i) = string(i+empty:i+empty)
40      continue
        do 50 i = ls-empty+1,ls
          string(i:i) = ' '
50      continue
      endif

      return
      end 

C-----------------------------------------------------------------------
      SUBROUTINE SHIFTR(STRING)
C-----------------------------------------------------------------------
C     Date               : 26/10/95 
C     Purpose            : Shift STRING to the right as far as possible
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER   i,ls,empty

      CHARACTER *(*) STRING
C-----------------------------------------------------------------------
      LS  = LEN (STRING)

      empty = 0
      i     = ls
30    if (string(i:i).eq.' ') then
        empty = empty+1
        i     = i-1
        if (i.ge.1) goto 30
      endif

      if (empty.gt.0) then
        do 40 i = ls,empty+1,-1
          string(i:i) = string(i-empty:i-empty)
40      continue
        do 50 i = 1,empty
          string(i:i) = ' '
50      continue
      endif

      return
      end

C ----------------------------------------------------------------------
      INTEGER FUNCTION STEPNR(ARRAY,LENGTH,X)
C ----------------------------------------------------------------------
C --- Find stepnumber 
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER I,LENGTH
      REAL*8  X,ARRAY(LENGTH)
C ----------------------------------------------------------------------
      IF (ARRAY(1).GE.X) GOTO 30

      DO 10 I = 2,LENGTH
        IF (ARRAY(I).GT.X)          GOTO 20 
        IF (ARRAY(I).LT.ARRAY(I-1)) GOTO 20
10    CONTINUE  
C --- array fully filled, argument larger than last X in array
      STEPNR = LENGTH
      RETURN
C --- array partly filled, argument larger than last X in array or
C --- argument between first and last X in table
20    STEPNR = I-1
      RETURN
C --- argument less or equal to first X in array
30    STEPNR = 1
      RETURN
      END


