************************************************************************
** FILE:
**    SWCOMLIB.FOR - common program code for SWAP and SWAPS programs
**
** FILE IDENTIFICATION:
**    $Id: SWCOMLIB.FOR 1.9.1.11 2002/09/02 08:53:10Z Kroes Exp $
** DESCRIPTION:
**    This file contains subroutines of SWAP which are similar in SWAPS.
**    The program units in this file are of the 3rd level and are sorted
**    alphabetically.
**
**    The following program units are in this file:
**        BOCOBOT, BOCODRB, BOCODRE, BOCOTOP, CALCGWL, CRACK, DIVDRA,
**        DMCNODE, HCONODE, HEADCALC, INITSOL, OUTDRF, OUTSWB,
**        PRHNODE, QHTAB, SLTBAL, SOLUTE, THENODE, TRIDAG, UPDATE, 
**        WATCON, WBALLEV, WLEVBAL, WLEVST, WSTLEV
************************************************************************

C ----------------------------------------------------------------------
      SUBROUTINE BOCOBOT (SWBOTB,GWL,GWLTAB,T,H,Z,LAYER,
     & FLGENU,COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS,THETHI,
     & DMCH,DMCC,KTABLE,KSE99,KSAT,FMOBIL,THETA,DIMOCA,K,KMEAN,
     & QBOT,QBOTAB,AQAVE,AQAMP,AQOMEG,AQTAMX,RIMLAY,COFQHA,COFQHB,
     & HGITAB,NUMNOD,SWMOBI,FM1,PF1,SLOPFM,NODGWL,SWDRA,QDRTOT,
     & QDRA,NRLEVS,QDRAIN,HDRAIN,SHAPE,GWLINP,RECIRES,AQATAB)
C ----------------------------------------------------------------------
C     Date               : 4/2/2000
C     Purpose            : determines boundary conditions at the bottom
C                          of the soil profile at time T
C     Subroutines called : -
C     Functions called   : AFGEN,THENODE,DMCNODE,HCONODE
C     File usage         : -
C     SWAP/SWAPS differences: Use timestep T+1 instead of T for inter-
C                             polation GWL
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C ---  global
       INTEGER SWBOTB,LAYER(MACP),THETLO(MAHO),THETHI(MAHO),NUMNOD
       INTEGER SWMOBI,NODGWL,SWDRA,NRLEVS

       REAL*8  GWL,GWLTAB(732),T,Z(MACP),QBOT,QBOTAB(732),HGITAB(732)
       REAL*8  FMOBIL(MACP),FM1(MAHO),PF1(MAHO),SLOPFM(MAHO),QDRTOT
       REAL*8  AFGEN,AQAMP,AQAVE,AQOMEG,AQTAMX,COFQHA,COFQHB,RIMLAY
       REAL*8  QDRA(5,MACP),QDRAIN(5),HDRAIN,SHAPE,GWLINP
       REAL*8  RECIRES,AQATAB(732),qreg

       REAL*8  H(MACP),KMEAN(MACP+1),THENODE,DMCNODE,HCONODE
       REAL*8  COFGEN(8,MAHO),HTABLE(MAHO,99),KTABLE(MAHO,99)
       REAL*8  ALFAMG(MACP),THETAR(MACP),THETAS(MACP),DMCC(MAHO,99)
       REAL*8  DMCH(MAHO,99)
       REAL*8  KSE99(MAHO),KSAT(MAHO),THETA(MACP),DIMOCA(MACP),K(MACP+1)

       LOGICAL FLGENU(MAHO)

	INTEGER GetErrorStatus
	EXTERNAL GetErrorStatus
C ----------------------------------------------------------------------
C --- local
      INTEGER  node,i,lay,level

      REAL*8   deepgw,hresis,hmax,qdrnode(macp),fluxtotal,ratio,gwlmean
C ----------------------------------------------------------------------
C --- interpolation between daily values of given groundwaterlevel
      if (swbotb.eq.1) then
        if (nodgwl.le.numnod) then
          hresis = gwl - real(h(numnod)) - real(z(numnod))
          hmax   = 0.5*(-z(numnod))
          hresis = min(hresis,hmax)
          hresis = max(hresis,-hmax)
        else
          hresis = 0.0
        endif

        gwlinp = AFGEN (GWLTAB,732,T)
C ----------------------------------------------------------------------

c --- conversion to pressure head at bottom of profile
        h(numnod)      = gwlinp-z(numnod)-hresis
      endif

C --- interpolation between daily values of given flux
      if (swbotb.eq.2) qbot = AFGEN (QBOTAB,732,T)

C --- seepage or infiltration from/to deep groundwater
      if (swbotb.eq.3) then
        gwlmean = hdrain + shape*(gwl - hdrain)
        deepgw = aqave + aqamp*cos(aqomeg*(t - aqtamx))
        qbot   = (deepgw - gwlmean)/rimlay
      endif

C --- flux calculated as function of h
      if (swbotb.eq.4) qbot = cofqha*exp(cofqhb*abs(gwl))

C --- interpolation between daily values of given pressurehead
      if (swbotb.eq.5) then
        h(numnod)      = AFGEN (HGITAB,732,T)
        lay            = layer(numnod)
        i              = numnod
        if (SWMOBI.eq.1) then
          if (h(i).gt.-1.0d0) then
            fmobil(i) = fm1(lay)+(0.0-pf1(lay))*slopfm(lay)
          else
            fmobil(i) = fm1(lay)+(log10(abs(real(h(i))))-pf1(lay))*
     &                  slopfm(lay)
          endif
          fmobil(i) = min (1.0d0,fmobil(i))
          fmobil(i) = max (0.3d0,fmobil(i))
          thetar(i) = cofgen(1,lay)*fmobil(i)
          thetas(i) = cofgen(2,lay)*fmobil(i)
        endif
        theta(numnod)  = thenode (numnod,h(numnod),LAYER,FLGENU,
     &                   COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS)
        dimoca(numnod) = dmcnode (numnod,h(numnod),LAYER,FLGENU,COFGEN,
     &                   THETLO,THETHI,DMCH,DMCC,THETAS,THETAR,
     &                   ALFAMG,SWMOBI,FM1,PF1,SLOPFM,HTABLE,THETA)
        k(numnod)      = hconode (numnod,theta(numnod),LAYER,FLGENU,
     &                   COFGEN,THETAR,THETAS,THETLO,THETHI,KTABLE,
     &                   KSE99,KSAT,SWMOBI,FMOBIL)
        kmean(numnod)  = 0.5*(k(numnod)+k(numnod-1))
	  if(GetErrorStatus().ne.0) return
      endif

C --- zero flux at the bottom
      if (swbotb.eq.6) qbot = 0.0

C --- free drainage
      if (swbotb.eq.7) qbot = -1.0 * kmean(numnod+1)

C --- lysimeter with free drainage
      if (swbotb.eq.8) qbot = 0.0

C --- combined Cauchy-Neuman
      if (swbotb.eq.9) then
        gwlmean = hdrain + shape*(gwl - hdrain)
        aqave   = AFGEN (AQATAB,732,T)
        qreg    = AFGEN (QBOTAB,732,T)
        qbot    = ReciRes*(aqave - gwlmean) + qreg
      endif

C --- restrict bottom and drainage flux if necessary
      if (gwl.lt.998.0d0 .and. nodgwl.lt.numnod .and. swbotb.le.6) then

        if (swdra.ge.1) then

C --- calculate total drainage flux of each node
          do 230 node = 1,numnod
            qdrnode(node) = 0.0
 230      continue
          qdrtot = 0.0

          do 250 node = nodgwl,numnod
            do 270 level = 1,nrlevs
              qdrnode(node) = qdrnode(node) + qdra(level,node)
 270        continue
            qdrtot = qdrtot + qdrnode(node)
 250      continue
        endif
        fluxtotal = qbot - qdrtot

C --- search for less conductive layers and restrict fluxes if necessary
        do 300 node = nodgwl,numnod
          lay = layer(node)
          if (fluxtotal .lt. -ksat(lay)) then
            ratio = -ksat(lay)/fluxtotal*0.8
            fluxtotal = 0.0
            do 320 i = node,numnod
              do 340 level=1,nrlevs
                qdra(level,i) = ratio * qdra(level,i)
 340          continue
              qdrnode(i) = ratio * qdrnode(i)
              fluxtotal = fluxtotal - qdrnode(i)
 320        continue
            qbot = ratio * qbot
            fluxtotal = fluxtotal + qbot + qdrnode(node)
          else
            fluxtotal = fluxtotal + qdrnode(node)
          endif
 300    continue
      endif      

C --- in case of drainage always calculate total drainage flux
      if (swdra.ge.1) then
        do 350 level = 1,nrlevs
          qdrain(level) = 0.0
 350    continue
        qdrtot = 0.0
        do 360 level = 1,nrlevs
          do 370 node = 1,numnod
             qdrain(level) = qdrain(level) + qdra(level,node)
 370      continue
          qdrtot = qdrtot + qdrain(level)
 360    continue
      endif

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE BOCODRB (DRAMET,GWL,ZBOTDR,POND,BASEGW,L,QDRAIN,IPOS,
     &  KHTOP,KHBOT,KVTOP,KVBOT,ENTRES,WETPER,ZINTF,GEOFAC,SWDTYP,
     &  OWLTAB,T,SWALLO,DRARES,INFRES,QDRTAB,NRLEVS)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : determines the values of the boundary       
C                          conditions at the lateral side of the soil  
C                          profile : basic drainage routine                    
C     Subroutines called : -                                           
C     Functions called   : AFGEN                                     
C     File usage         : -                                           
C     SWAP/SWAPS differences: None
C ----------------------------------------------------------------------  
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER DRAMET,IPOS,SWDTYP(5),SWALLO(5),NRLEVS

      REAL*8  GWL,ZBOTDR(5),POND,BASEGW(2),L(5),QDRAIN(5),KHTOP,KHBOT
      REAL*8  KVTOP,KVBOT,ENTRES,WETPER(5),ZINTF,GEOFAC,T
      REAL*8  DRARES(5),INFRES(5),QDRTAB(50),AFGEN,OWLTAB(5,2*MAOWL)
C ----------------------------------------------------------------------
C --- local
      INTEGER i,lev

      REAL*8  dh,zimp,dbot,pi,totres,x,fx,eqd,rver,rhor,rrad
      REAL*8  TEMPTAB(2*MAOWL)

      LOGICAL FLDRY

      PARAMETER (pi=3.14159d0)

	character*80 errTxt,errSrc
C ----------------------------------------------------------------------
      errSrc = 'BOCODRB'

C --- drainage flux calculated according to Hooghoudt or Ernst
      if (dramet.eq.2) then
        dh    = gwl-zbotdr(1)
        if (abs(gwl).lt.1.0d-7) dh = dh + pond

C --- contributing layer below drains limited to 1/4 L
        zimp  = max (basegw(1),zbotdr(1)-0.25*L(1))
        dbot  = (zbotdr(1)-ZIMP)  
        if (dbot.lt.0.0) then
	    errTxt = 'DBOT negative'
	    call SetFatalError(9999,errTxt,errSrc)
	    return
	  endif
	  	
C --- no infiltration allowed
        if (dh.lt.1.0d-10) then
          qdrain(1) = 0.0
          return
        endif

C --- case 1: homogeneous, on top of impervious layer
        if (ipos.eq.1) then

C --- calculation of drainage resistance and drainage flux
          totres    = l(1)*l(1)/(4*khtop*abs(dh)) + entres
          qdrain(1) = dh/totres 

C --- case 2,3: in homogeneous profile or at interface of 2 layers
        elseif (ipos.eq.2.or.ipos.eq.3) then

C --- calculation of equivalent depth
          x = 2*pi*dbot/l(1)
          if (x.gt.0.5d0) then
            fx = 0.0
            do 10 i = 1,5,2
              fx = fx + (4*exp(-2*i*x))/(i*(1.0d0-exp(-2*i*x)))
   10       continue
            eqd = pi*l(1)/8 / (log(l(1)/wetper(1))+fx)
          else
            if (x.lt.1.0d-6) then
              eqd = dbot
            else
              fx  = pi**2/(4*x) + log(x/(2*pi))
              eqd = pi*l(1)/8 / (log(l(1)/wetper(1))+fx)
            endif
          endif 
          if (eqd.gt.dbot) eqd = dbot

C --- calculation of drainage resistance & drainage flux
          if (ipos.eq.2) then
            totres = l(1)*l(1)/(8*khtop*eqd+4*khtop*abs(dh)) + entres 
          elseif (ipos.eq.3) then
            totres = l(1)*l(1)/(8*khbot*eqd+4*khtop*abs(dh)) + entres
          endif
          qdrain(1) = dh/totres 

C --- case 4: drain in bottom layer
        elseif (ipos.eq.4) then
          if (zbotdr(1).gt.zintf) then
	      errTxt = 'check ZINTF and ZBOTDR'
		  call SetFatalError(9999,errTxt,errSrc)
	      return
	    endif 
		rver      = max (gwl-zintf,0.0d0) / kvtop +
     &                (min (zintf,gwl) -zbotdr(1))/kvbot
          rhor      = l(1)*l(1)/(8*khbot*dbot) 
          rrad      = l(1)/(pi*sqrt(khbot*kvbot)) * log(dbot/wetper(1))
          totres    = rver+rhor+rrad+entres
          qdrain(1) = dh/totres 

C --- case 5 : drain in top layer
        elseif (ipos.eq.5) then
          if (zbotdr(1).lt.zintf) then
		  errTxt = 'check ZINTF and ZBOTDR'
		  call SetFatalError(9999,errTxt,errSrc)
	      return
          endif
		rver      = (gwl-zbotdr(1))/kvtop
          rhor      = l(1)*l(1) / (8*khtop*(zbotdr(1)-zintf) +
     &                             8*khbot*(zintf-zimp))
          rrad      = l(1)/(pi*sqrt(khtop*kvtop))*log((geofac*
     &                (zbotdr(1)-zintf))/wetper(1))
          totres    = rver+rhor+rrad+entres
          qdrain(1) = dh/totres
        endif

C --- drainage flux calc. using given drainage/infiltration resistance
      elseif (dramet.eq.3) then
        do 1000 LEV = 1,NRLEVS
          FLDRY = .FALSE.
          if (swdtyp(lev).eq.1) then
            dh = gwl-zbotdr(lev)
          elseif (swdtyp(lev).eq.2) then

C --- first copy to 1-dimensional table TEMPTAB
            do 12 I = 1,2*MAOWL 
              TEMPTAB(I) = OWLTAB(LEV,I)
   12       continue
            if (afgen (TEMPTAB,2*MAOWL,t+1.0d0)-zbotdr(lev).lt.1.0d-3) 
     &      FLDRY = .TRUE.  
            dh = gwl- afgen (TEMPTAB,2*MAOWL,t+1.0d0)
            if (FLDRY) dh = gwl-zbotdr(lev)
          endif
          if (abs(gwl).lt.1.0d-7) dh = dh + pond
          if (dh.ge.0.0) then

C --- drainage
            qdrain(lev) = dh/drares(lev)
            if (swallo(lev).eq.2) qdrain(lev) = 0.0
          else

C --- infiltration
            qdrain(lev) = dh/infres(lev)
            if (swallo(lev).eq.3.or.FLDRY) qdrain(lev) = 0.0
          endif

 1000   continue

C --- drainage flux from table with gwlevel - flux data pairs
      elseif (dramet.eq.1) then
        qdrain(1) = afgen (qdrtab,50,abs(gwl))
      endif

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE BOCODRE (SWSEC,SWSRF,NRSRF,NRPRI,GWL,ZBOTDR,TALUDR,
     & WIDTHR,POND,PONDMX,SWDTYP,DT,WLS,WLP,DRAINL,L,RDRAIN,RINFI,
     & RENTRY,REXIT,GWLINF,WETPER,QDRAIN,QDRD,T,IMPEND,NMPER,WSCAP,WST,
     & SWNRSRF,RSURFDEEP,RSURFSHALLOW)
C ----------------------------------------------------------------------
C     Date               : 5/5/2001
C     Purpose            :        
C --- Calculate drainage/infiltration fluxes for all levels: qdrain(level).
C --- Summate fluxes to secondary system: qdrd
C --- Given present storage (wst), qdrd and wscap(imper), check if the
C --- system falls dry. If so reduce drainage fluxes proportionally in 
C --- such a way that total drainage flux equals available amount 
C --- (= wst + wscap).
C     Subroutines called : -                                           
C     Functions called   : -                                     
C     File usage         : -  Error handling
C ----------------------------------------------------------------------  
      IMPLICIT NONE

C --- global
      INTEGER SWSEC,SWSRF,NRSRF,NRPRI,SWDTYP(5),IMPEND(10),NMPER,SWNRSRF

      REAL*8  GWL,ZBOTDR(5),TALUDR(5),WIDTHR(5),POND,PONDMX,WLS,WLP
      REAL*8  DRAINL(5),L(5),WETPER(5),RINFI(5),REXIT(5),GWLINF(5),DT,T
      REAL*8  RDRAIN(5),RENTRY(5),QDRAIN(5),QDRD,WST
      REAL*8  WSCAP(10), rsurfdeep, rsurfshallow

C --- local
      INTEGER level,imper
      REAL*8  dh,qdrdm,qdratio,swdepth,swexbrd,dvmax,wstmax,wl,rd,re

C --- error handling
      character*80 ErrTxt,ErrSrc

      save
C ----------------------------------------------------------------------

C --- Intialize Error Handling (Source name)
      ErrSrc = 'Bocodre'

C --- summate fluxes for use by swballev and swlevbal
      qdrd = 0.0
      do 500 level = 1,nrsrf

C --- surface water level
        if (swsrf.ge.2) then
          if (level.gt.nrpri) then
            wl = wls
          else
            wl = wlp
          endif
        endif

C --- drainage fluxes are set to zero if both groundwater level and surface 
C     water level are above ponding sill (so the nonzero drainage flux 
C     is only computed if either the gwl or the wl is below pondmx)
        if (wl .lt. pondmx .or. gwl .lt. pondmx) then

C --- channel is active medium if either groundwater or surface water
C     level is above channel bottom
          if (gwl.gt.(zbotdr(level)+0.001d0) .or. 
     &        wl.gt.(zbotdr(level)+0.001d0)) then
            if (wl .le. (zbotdr(level)+0.001d0).or.swsrf.eq.1) then

C --- only groundw. level above channel bottom; bottom is dr. base
              drainl(level) = zbotdr(level)

C --- wetted perimeter only computed for open channels
C     (for drains it is input) 
              if (swdtyp(level) .eq. 0) then
                wetper(level) = widthr(level)
              endif
            else

C --- surface water level above channel bottom
              drainl(level) = wl
              if (swdtyp(level) .eq. 0) then
                swdepth = wl-zbotdr(level)
                swexbrd = (wl-zbotdr(level))/taludr(level)
                wetper(level) = widthr(level) +
     &            2*sqrt(swdepth**2 + swexbrd**2) 
              endif
            endif

C --- drainage flux (cm/d)
            dh = gwl - drainl(level)
            if (gwl.gt.-0.1d0) dh = dh+pond
            if (dh.lt.0.0 .and.gwl.lt.gwlinf(level)) then
              dh = gwlinf(level)-drainl(level)
            endif

            if (dh.gt.0.0) then
              rd = rdrain(level)
              re = rentry(level)
C ---               surface drainage (stofzuiger)
              if ((level.eq.nrsrf).and.(swnrsrf.eq.1)) then
                rd = rsurfdeep - dh
			  rd = max(rd,rsurfshallow)
			endif
            else
              rd = rinfi(level)
              re = rexit(level)
            endif 
            if (swdtyp(level).eq.0) then
              qdrain(level) = dh / ((rd + re*l(level)/wetper(level)))
            else
              qdrain(level) = dh/rd
            endif 
          else
            if (swdtyp(level) .eq. 0) then
              wetper(level) = 0.0
            endif
            dh = 0.0
            qdrain(level) = 0.0
          endif
c
	  else
	    qdrain(level) = 0.0
        endif
c
        if (swsrf.ge.2.and.level.gt.nrpri) then

C --- qdrd is total flux to or from secondary system
          qdrd = qdrd + qdrain(level)
        endif
 500  continue

C ----------------------------------------------------------------------
C --- check for system falling dry (only for swsec = 2):
      if (swsec .eq. 1) return
      if (swsrf.eq.1) then
        do 10 level = 1,nrsrf
          if (qdrain(level).lt.0.0) then
            qdrain(level) = 0.0
          endif
 10     continue
      elseif (swsrf.ge.2) then

C --- determine which management period the model is in:
        imper   = 0
 800    imper   = imper + 1

C ---   Error handling
        if (imper .gt. nmper) then
          errTxt = 'sw-management periods'
		call SetFatalError(9999,errTxt,ErrSrc)
          return
        endif

        if (int(T+1.0d0) .gt. impend(imper)) goto 800

C ---   determine whether the system will become empty
        dvmax  = (qdrd + wscap(imper)) * dt
        wstmax = wst + dvmax

        if (wstmax .lt. 0.0) then
C ---     storage decreases to below zero, then the surface water system 
C         falls dry; make the total infiltration exactly equal to the
C         available amount:
          qdrdm = - (wst + wscap(imper)*dt) / dt
          qdratio = qdrdm / qdrd

C ---     Error handling
          if (qdratio .gt. 1.0d0 .or. qdratio .lt. 0.0d0) then
            errTxt = 'sw-management periods'
            call SetFatalError(9999,ErrTxt,ErrSrc)
            return
          endif

          do 820 level=1+NRPRI,nrsrf
            qdrain(level) = qdrain(level)*qdratio
 820       continue
          qdrd = qdrdm
        endif
      endif

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE BOCOTOP (POND,REVA,NIRD,DT,QTOP,QBOT,QDRTOT,
     & KMEAN,K,H,DISNOD,LAYER,FLGENU,COFGEN,THETA,THETIM,DZ,
     & HTABLE,THETLO,THETHI,KTABLE,ALFAMG,FTOPH,ARFLUX,THETAR,
     & THETAS,KSE99,KSAT,SWMOBI,FMOBIL,VOLSAT,VOLACT,ACRACK,QCRACK,
     & FLTSAT,HSURF,DTMIN,WTOPLAT,FLREVA,NUMNOD,QROSUM,
     & QCOMP,FPEGWL,NPEGWL,Q,SWBOTB,GWLINP,hAtm,FlDecDtMin,revadeep)
C ----------------------------------------------------------------------
C     Date               : 4/2/2000
C     Purpose            : determines the values of the boundary       
C                          conditions at the top of the profile        
C     Formal parameters  :                                             
C     Subroutines called : -                                           
C     Functions called   : hconode,thenode                                 
C     File usage         : -                                           
C     SWAP/SWAPS differences: 
C     - Assignment HATM different in SWAP/SWAPS
C ----------------------------------------------------------------------
C
C --- SWAPS NOTE on arguments:
C
C     SWAPS external name       Local name
C     -------------------       ----------
C     revatop                   REVA
C     nrai                      ARFLUX
C     flrevatop                 FLREVA
C
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER LAYER(MACP),THETLO(MAHO),THETHI(MAHO),SWMOBI,NUMNOD,NODE
      INTEGER NPEGWL,SWBOTB

      REAL*8  POND,REVA,NIRD,DT,QTOP,QBOT,QDRTOT,DISNOD(MACP+1),WTOPLAT
      REAL*8  FMOBIL(MACP),ARFLUX,ACRACK,QCRACK,HSURF,DTMIN,QROSUM
      REAL*8  QCOMP,DZ(MACP),GwlInp,hAtm
      REAL*8  KSURF,HCONODE,KMEAN(MACP+1),K(MACP+1),H(MACP)
      REAL*8  COFGEN(8,MAHO),THETA(MACP),THETIM(MAHO)
      REAL*8  HTABLE(MAHO,99),KTABLE(MAHO,99),ALFAMG(MACP)
      REAL*8  THETAR(MACP),THETAS(MACP),THENODE,KSE99(MAHO),KSAT(MAHO)
      REAL*8  VOLACT,VOLSAT,Q(MACP+1)

      LOGICAL FLGENU(MAHO),FTOPH,FLTSAT,FLREVA,FPEGWL
      LOGICAL FlDecDtMin

      REAL*8  revadeep

C ----------------------------------------------------------------------
C --- local variables
      INTEGER i,lay
      REAL*8  inflow,qsurf,vair,Emax,Imax,qtop1

	INTEGER GetErrorStatus
	EXTERNAL GetErrorStatus

      save
C ----------------------------------------------------------------------


C --- Calculate hydraulic conductivity corresponding with hAtm
      if (hAtm.lt.0.0) Then
         ksurf = hconode (1,THENODE(1,DBLE(HATM),LAYER,FLGENU,
     &        COFGEN,HTABLE,THETLO,ALFAMG,THETAR,THETAS),LAYER,
     &        FLGENU,COFGEN,THETAR,THETAS,THETLO,THETHI,KTABLE,
     &        KSE99,KSAT,SWMOBI,FMOBIL)
         if (GetErrorStatus().ne.0) return
	else

C --- This only occurs if RH is 100% in SWAPS, never used for SWAP
         kSurf = k(1)
      endif

C --- determine potential flux at the top of the profile
      flreva = .false.

C --- save qtop from previous time step
      qtop1 = qtop
      if (pond.gt.1.0d-6) then
        qtop = reva-(pond-wtoplat)/dt-(ARFLUX+NIRD)*(1.0d0-acrack)+qcomp
      else
        qtop = reva-(ARFLUX+NIRD)*(1.0d0-acrack)+qcomp
      end if

c --- decrease time step at transition evaporation --> large infiltration
      if (qtop1 .gt. 0.001d0 .and. qtop .lt. -1.0d0 .and. 
     &  pond.lt.1.0d-7 .and. dt .gt. (dtmin+1.d-9)) then
        FlDecDtMin = .true.        
	  return
      endif        

C --- maximum evaporation rate according to Darcy
      kmean(1) = 0.5*(ksurf+k(1))
      Emax = kmean(1) * (h(1)-hatm) / disnod(1) - kmean(1)

C --- maximum infiltration rate according to Darcy
      kmean(1) = 0.5*(ksat(1)+k(1))
      Imax = kmean(1) * (pond-h(1)) / disnod(1) + kmean(1)
      Imax = max(ksat(1),Imax)

C --- potential flux at soil surface
      qsurf = min(qtop,Emax)
      qsurf = max(qsurf,-Imax)

C --- determine inflow
      if (fpegwl.and.npegwl.gt.0) then

C --- perched groundwater level exists
        inflow = (- qsurf + q(npegwl+1)) * dt
      else

C --- just phreatic groundwater level
        inflow = (qbot - qsurf - qrosum - revadeep - qdrtot + qcrack)*dt

      endif

C --- set flux boundary condition
      ftoph = .false.

c --- determine whether soil profile is saturated
      fltsat = .true.
      do 100 node = 1,numnod
        if (h(node) .lt. 0.0) fltsat = .false.
 100  continue      

      if (fltsat) then

c --- soil profile was saturated
        if (inflow .gt. -1.0d-6) then

C --- profile remains saturated, head boundary condition
          ftoph    = .true.
          hsurf    = inflow
          kmean(1) = ksat(1)
        else

C --- profile becomes unsaturated
          if (dt .gt. (dtmin+0.001d0)) then

C --- decrease time step for numerical stability!
            FlDecDtMin = .true.
            return
          endif

          if (qtop .ge. 0.0) then

c --- maximum evaporation flux soil
            if (qtop.gt.Emax) then
              ftoph  = .true.
              if (reva.gt.0.0) flreva = .true.
              hsurf  = hAtm
              kmean(1) = 0.5*(ksurf+k(1))
            endif             
          else

c --- maximum infiltration flux soil
            if ((qtop .lt. -ksat(1)) .and. (qtop .lt. (-Imax))) then 
              ftoph = .true.
              hsurf = pond
              kmean(1) = 0.5*(ksat(1)+k(1))
            endif
          endif
        endif
      else

c --- soil profile was unsaturated

c --- air volume in case of perched groundwater level
        if (fpegwl.and.npegwl.gt.0) then
          vair = 0.0
          do 120 i = 1,npegwl
            lay    = layer(i)
            vair = vair + thetas(i) - theta(i) - 
     &                        (1.0d0-fmobil(i))*thetim(lay)*dz(i)
 120      continue

c --- air volume in case of phreatic groundwater level
	  else
          vair = volsat-volact
	  endif

        if (inflow.ge.vair) then

C --- profile becomes saturated
          ftoph = .true.
          if (swbotb.eq.1) then
            hsurf = gwlinp
          else
            hsurf = inflow - vair
          endif
          kmean(1) = ksat(1)
        else

C --- profile remains unsaturated
          if (qtop .ge. 0.0) then

C --- maximum evaporation flux soil
            if (qtop .gt. Emax) then
              ftoph = .true.
              if ((Emax.gt.0.0).and.(reva.gt.0.0)) flreva = .true.
              hsurf = hAtm
              kmean(1) = 0.5*(ksurf+k(1))
            endif             
          else

C --- maximum infiltration flux soil
            if ((qtop .lt. -ksat(1)) .and. (qtop .lt. (-Imax))) then 
              ftoph = .true.
              hsurf = pond
              kmean(1) = 0.5*(ksat(1)+k(1))
            endif
          endif
        endif
      endif
     
      return
      end


C ----------------------------------------------------------------------
      SUBROUTINE CALCGWL(GWL,GWLBAK,NUMNOD,H,Z,DZ,SWBOTB,POND,NODGWL,
     &           FPEGWL,NPEGWL,PEGWL,GWLTAB,T)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : search for the watertable and perched
C                          watertable(s) (if existing).
C     Formal parameters  :                                             
C     Subroutines called : -                                           
C     Functions called   : -                                     
C     File usage         : -
C     SWAP/SWAPS differences: - Error handling
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER  NUMNOD,SWBOTB,NODGWL,NPEGWL
     
      REAL*8   GWL,GWLBAK(4),Z(MACP),DZ(MACP),POND,T,PEGWL,GWLTAB(732)
      REAL*8   H(MACP)
      LOGICAL  FPEGWL

C ----------------------------------------------------------------------
C --- local
      INTEGER  node,i
      REAL*8   Afgen
      LOGICAL  flsat

C --- error handling
      character*80 ErrTxt,ErrSrc

      save
C ----------------------------------------------------------------------
C --- Intialize Error Handling (Source name)
      errSrc = 'CalcGWL'

C --- set initial values
      gwl       = 999.0d0
      flsat     = .false.
      nodgwl    = numnod+1
      fpegwl    = .false.

C --- search for ground watertable
      if (h(numnod).ge.0.0d0) then
        flsat  = .true.
      endif
      do 200 node = numnod-1,1,-1
          if (flsat.and.h(node).lt.0.0d0) then
            gwl = (z(node+1) + h(node+1) / (h(node+1)-h(node))
     $            * 0.5 * (dz(node) + dz(node+1))) 
            flsat   =.false.
            nodgwl  = node
          endif
 200  continue

C --- groundwater level near soil surface
      if (flsat) then
        if (pond .lt. 1.d-8) then
          gwl = z(1) + h(1)
        else
          gwl = pond
        endif
        nodgwl = 1
      endif 
 
c --- search for perched groundwater table
      do 300 node = numnod,2,-1

c ---   first search for transition to saturated part 
        if ((.not.fpegwl) .and. (h(node).lt.0.0d0)
     &	                .and. (h(node-1).gt.0.0d0)) then            
          do 400 i = node-1,2,-1

c ---       next search for transition to unsaturated part 
            if ((.not.fpegwl) .and. (h(i).gt.0.0d0)
     &	                    .and. (h(i-1).lt.0.0d0)) then            
              fpegwl  =.true.
              npegwl  = i - 1
              pegwl   = (z(i-1)*h(i)-z(i)*h(i-1))/(h(i)-h(i-1))
            endif
 400      continue
          if (.not.fpegwl) then
            fpegwl = .true.
            npegwl = 0
            pegwl  = pond
          endif
        endif

 300  continue

C --- set gwl in case gwl is prescribed below soil profile
      if (swbotb.eq.1.and.gwl.gt.998d0) gwl = afgen(gwltab,732,t)

C --- warning gwl below profile
      if ((swbotb.eq.3.or.swbotb.eq.4).and.gwl.gt.998.0d0) then
        errTxt = 'Groundwater table below lower boundary'
	  call SetFatalError(9999,errTxt,errSrc)
        return
      endif

C --- updating registration of last four groundwater levels, for noting 
C     oscillation
        gwlbak(1) = gwlbak(2)
        gwlbak(2) = gwlbak(3)
        gwlbak(3) = gwlbak(4)
        gwlbak(4) = gwl

      return
      end


C ----------------------------------------------------------------------
      subroutine CRACK (layer,nncrack,thetsl,theta,shrina,shrinb,shrinc,
     & acrack,wtoplat,geomf,pond,numnod,adsflu,thetcr,z,dz,qcrack,
     & moisr1,moisrd,crackw,flgenu,cofgen,thetar,thetas,thetlo,thethi,
     & ktable,kse99,ksat,swmobi,fmobil,h,diampol,disnod,rapdra,rapcoef,
     & cralev)
C ----------------------------------------------------------------------
C      Date               : 29/9/99
C      Purpose            : infiltration into cracks, absorption to     
C                           soil matrix, rapid drainage                 
C      Subroutines called :
C      Functions called   :
C      File usage         :
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER swmobi,nncrack,layer(MACP),numnod,thetlo(MAHO)
      INTEGER thethi(MAHO)

      REAL*8  shrina,shrinb,shrinc,acrack,wtoplat,geomf,pond,cralev
      REAL*8  adsflu(MACP),thetcr(MAHO),z(MACP),dz(MACP),qcrack,moisr1
      REAL*8  crackw,fmobil(MACP),diampol,disnod(MACP+1),rapdra,rapcoef
      REAL*8  moisrd

      REAL*8  thetsl(MAHO),theta(MACP),hconode,cofgen(8,MAHO)
      REAL*8  ktable(MAHO,99)
      REAL*8  ksat(MAHO),kse99(MAHO),thetar(MACP),thetas(MACP),h(MACP)

      LOGICAL flgenu(MAHO)
C ----------------------------------------------------------------------
C --- local
      INTEGER lay,node

      REAL*8  vsolid,moisr,voidr,vapore,vshrink,subsid,vcrack
      REAL*8  totalv,conduc,ratio,hcrack,cradepth

	INTEGER GetErrorStatus
	EXTERNAL GetErrorStatus

C ----------------------------------------------------------------------

C --- shrinkage volume and area crack at soil surface
      lay = layer(nncrack)
      if (theta(nncrack) .gt. (thetsl(lay)-0.01D0)) then
        acrack  = 0.0
        wtoplat = 0.0
      else
        vsolid =1.0d0 - thetsl(lay)
        moisr  = theta(nncrack) / vsolid
        if (moisr .lt. moisr1) then
          voidr = shrina / exp(shrinb*moisr) + shrinc * moisr
        else
          voidr = moisr1 + moisrd
        endif
        vapore = voidr * vsolid
        vshrink =thetsl(lay) - vapore
        subsid = 1.0d0 - (1.0d0 - vshrink) ** (1.0 / geomf)
        vcrack = vshrink - subsid
        acrack = vcrack / (1.0d0 - subsid)

C --- lateral flow due to ponding at the soil surface 
        wtoplat = pond
      endif

C --- initialize values
      do 50 node = 1,numnod
        adsflu(node) = 0.0
 50   continue
      totalv = 0.0
      qcrack = 0.0
      cralev = 0.0
      cradepth = 0.0

C --- determine depth of crack
      node = numnod
      lay  = layer(node)
 60   if (theta(node).gt.(thetcr(lay) - 1.0d-4) .and. node.gt.1) then
        node = node - 1
        lay  = layer(node)
        goto 60
      endif

C --- skip calculation when cracks are closed until soil surface
      if (node.eq.1 .and. theta(1).gt.(thetsl(1)-0.01D0)) goto 70
        cradepth = z(node) - 0.5*dz(node)

C --- crack volume for each node 
 65   if (theta(node) .gt. (thetsl(lay)-0.01D0)) then
        if (node .gt. 1) then
          node = node -1
          lay = layer(node)
          goto 65
        endif
      else
        vsolid = 1.0d0 - thetsl(lay)
        moisr = theta(node) / vsolid
        if (moisr .lt. moisr1) then
          voidr = shrina / exp(shrinb*moisr) + shrinc * moisr
        else
          voidr = moisr1 + moisrd
        endif
        vapore = voidr * vsolid
        vshrink = thetsl(lay) - vapore
        subsid = 1.0d0 - (1.0d0 - vshrink) ** (1.0 / geomf)
        vcrack = (vshrink - subsid)*dz(node)
        totalv = totalv + vcrack

c --- determine lateral infiltration flux from crack 
        if (totalv .lt. crackw) then

c --- compartment participates with full height 
          conduc = hconode (node,theta(node),LAYER,FLGENU,COFGEN,
     &           THETAR,THETAS,THETLO,THETHI,KTABLE,KSE99,KSAT,SWMOBI,
     &           FMOBIL)
	    if (GetErrorStatus().ne.0) return
          hcrack = z(node) - cradepth
          adsflu(node) = (- conduc*24*dz(node)*(h(node)-hcrack)/
     &                       (diampol*diampol))
          qcrack = qcrack + adsflu(node)
          if (node .gt. 1) then
            node = node -1
            lay = layer(node)
            goto 65
          endif
        else

c --- compartment participates with part of total height 
          ratio  = (crackw - (totalv - vcrack)) / vcrack
          cralev = z(node) - 0.5 * disnod(node+1) + ratio * dz(node)
          conduc =hconode (node,theta(node),LAYER,FLGENU,COFGEN,
     &           THETAR,THETAS,THETLO,THETHI,KTABLE,KSE99,KSAT,SWMOBI,
     &           FMOBIL)
          hcrack = z(node) -cradepth       
          adsflu(node) = (- conduc*24*dz(node)*(h(node)-cradepth)/
     &                       (diampol*diampol)* ratio)
          qcrack = qcrack + adsflu(node)
	    if (GetErrorStatus().ne.0) return
        endif
      endif

C --- rapid drainage
 70   rapdra = rapcoef * crackw

      return
      end


C ----------------------------------------------------------------------
      SUBROUTINE DIVDRA( NumComp, NumDrain,ThickComp,KSAT,LAYER,COFANI,
     &                   GWLEV, DistDrain, FluxDr,FluxDrComp)  
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : Simulation of lateral waterfluxes in the
C                          saturated zone. In two steps:
C                          1. Calculate bottom-boundaries of
C                             model_discharge_layers;
C                          2. Distribute drainage fluxes over each
C                             model_discharge_layer.
C                          
C     Subroutines called : -                                           
C     Functions called   : -                                     
C     File usage         : -
C     Differences SWAP/SWAPS: None
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

      INTEGER DrainSequence(5),Icomp,idr,iidr
      INTEGER idum,jdr,NumComWatLev,NumComBotDislay(5),NumActDrain
      INTEGER NumDrain,NumComp,LAYER(MACP)

      REAL*8  FacAniso,CondSatHorAv,CondSatVerAV,BotDisLay(5),ThickCum
      REAL*8  CumThickCondHor,CumThickCondVer,DistDrain(5),Dum
      REAL*8  FluxDr(5),FluxDrComp(5,MACP),FlowDrDisch(5),HelpFl(5)
      REAL*8  HelpTh,HelpTr,MaxDepthDislay(5),CondSatHor(MACP)
      REAL*8  CondSatVer(MACP),Small,ThickComp(MACP),ThickCompSatWatLev
      REAL*8  ThickCompBotDislay(5),Transmissivity(5),WatLevAv,GWLEV
      REAL*8  COFANI(MAHO)

      REAL*8  KSAT(MAHO)

      LOGICAL NonExistFluxDr

      DATA    Small /1.0d-10/
C ----------------------------------------------------------------------

C --- groundwater level converted to cm below surface level 
      WatLevAv = -1*MIN(GWLEV,0.0d0)

C --- calculation of CondSatHor and CondSatVer
      do 10 icomp = 1,NumComp
        CondSatHor(icomp) = KSAT(LAYER(icomp))
        CondSatVer(icomp) = KSAT(LAYER(icomp))*COFANI(LAYER(icomp))
 10   continue

C --- Initialisation and test whether any drainflux exists
      NonExistFluxDr = .true.
      do idr=1,NumDrain
         if( abs( FluxDr(idr) ) .gt. Small ) NonExistFluxDr = .false.
         do icomp = 1, NumComp
            FluxDrComp(idr,icomp) = 0.0
         enddo
	   DrainSequence(idr) = 0
      enddo      
      if( NonExistFluxDr ) return

C --- paragraph 2 
C --- Search for compartment with groundwaterlevel (NumComWatLev)
      NumComWatLev = 1
      ThickCum     = ThickComp(1)
      do while (WatLevAv.gt.ThickCum)
         NumComWatLev = NumComWatLev + 1
         ThickCum     = ThickCum + ThickComp(NumComWatLev)
      enddo

C --- saturated part (ThickComp_unsat) of compartment with waterlevel
      ThickCompSatWatLev = ThickCum - WatLevAv

C --- paragraph 3
C --- Calculate overall anisotropic factor model profile and 
C --- cumulative transmissivity as a function of depth
      CumThickCondHor = ThickCompSatWatLev * CondSatHor(NumComWatLev)
      CumThickCondVer = ThickCompSatWatLev / CondSatVer(NumComWatLev)
      ThickCum        = ThickCompSatWatLev

      do icomp=NumComWatLev+1,NumComp
         CumThickCondHor = CumThickCondHor + ThickComp(icomp) * 
     &                       CondSatHor(icomp)
         CumThickCondVer = CumThickCondVer + ThickComp(icomp) /
     &                       CondSatVer(icomp)
         ThickCum        = ThickCum + ThickComp(icomp)
      enddo
      
      CondSatHorAv      = CumThickCondHor / ThickCum
      CondSatVerAv      = ThickCum / CumThickCondVer
      FacAniso          = sqrt( CondSatVerAv / CondSatHorAv )

C --- paragraph 4      
C --- Calculate maximum depth per drainage system and discharge 
C --- flow rate per drainage system 
      do idr=1,NumDrain
         MaxDepthDislay(idr) = 0.25*DistDrain(idr)*FacAniso+WatLevAv

C ---    beperken tot SWAP profiel dikte
         MaxDepthDislay(idr) = MIN (MaxDepthDislay(idr),
     &                              Thickcum+WatLevAv)
      enddo

C --- paragraph 5      
C --- determine sequence of order of drainage systems 

      NumActDrain = 0
      do idr=1,NumDrain
         if ( abs(FluxDr(idr)) .gt. small ) then
            NumActDrain = NumActDrain+1
            DrainSequence(NumActDrain) = NumActDrain
            HelpFl(NumActDrain)        = DistDrain(idr)
         end if 
      end do
      
      do idr=1,NumActDrain-1
         do iidr = idr+1,NumActDrain
            if ( HelpFl(idr) .lt. HelpFl(iidr) )then
             dum                 = HelpFl(iidr)
             HelpFl(iidr)        = HelpFl(idr)
             HelpFl(idr)         = dum
             idum                = DrainSequence(iidr)
             DrainSequence(iidr) = DrainSequence(idr)
             DrainSequence(idr)  = idum
            endif
         enddo
      enddo       

      idr = DrainSequence(NumActDrain)
      FlowDrDisch(idr) = abs(FluxDr(idr)*HelpFl(idr))
      do iidr=NumActDrain-1,1,-1 
         idr = DrainSequence(iidr)
         jdr = DrainSequence(iidr+1)
         FlowDrDisch(idr) = FlowDrDisch(jdr) + 
     &                      abs(FluxDr(idr)*HelpFl(idr))
      enddo

C --- paragraph 6
C --- Bottom of 1st order model_discharge_layer

      idr = DrainSequence(1)
      Transmissivity(idr)     = CumThickCondHor
      BotDisLay(idr)          = ThickCum + WatLevAv
      NumComBotDislay(idr)    = NumComp
      ThickCompBotDislay(idr) = ThickComp(NumComp)           
      
C --- correction of BotDisLay(1) if D1 < 0.25 L \/(kv/kh) 
      if (abs(FluxDr(idr)).gt.Small) then
         if (BotDisLay(idr) .gt. MaxDepthDislay(idr) )then
            BotDisLay(idr) = MaxDepthDislay(idr)

C ---       determine adjusted transmissivity and compartment 
C ---       number which contains bottom of discharge layer
            NumComBotDislay(idr) = NumComWatLev
            HelpTh               = ThickCompSatWatLev
            Transmissivity(idr)  = ThickCompSatWatLev * 
     &                             CondSatHor(NumComWatLev)
            do while ( BotDisLay(idr)-WatlevAv .gt. HelpTh )      
               NumComBotDislay(idr) = NumComBotDislay(idr) + 1       
               HelpTh               = HelpTh + 
     &                                ThickComp(NumComBotDislay(idr))
               Transmissivity(idr)  = Transmissivity(idr) + 
     &                                ThickComp(NumComBotDislay(idr)) * 
     &                                CondSatHor(NumComBotDislay(idr))              
            enddo
            Transmissivity(idr) = Transmissivity(idr) - 
     &                          (HelpTh - BotDisLay(idr)) * 
     &                          CondSatHor(NumComBotDislay(idr))              

C ---       thickness of part of bottom compartment
            ThickCompBotDislay(idr) = ThickComp(NumComBotDislay(idr)) - 
     &                              (HelpTh - BotDisLay(idr)) 
         endif      
      else 
         BotDisLay(idr) = WatLevAv
      endif

C --- paragraph 7
C --- Bottom of 2nd and higher order model_discharge_layers 
C --- for drainage system of orders 2 to NumDrain (number of drains)         
      do iidr=2,NumActDrain
         idr = DrainSequence(iidr) 
         jdr = DrainSequence(iidr-1)
         Transmissivity(idr) = Transmissivity(jdr) * 
     &                         FlowDrDisch(idr) / FlowDrDisch(jdr) 

C --- bottom of discharge layer of order i
C     and thickness of bottom compartment, and
C     number of bottom compartment
         icomp  = NumComWatLev
         HelpTr = ThickCompSatWatLev * CondSatHor(icomp)
         HelpTh = ThickCompSatWatLev
         do while ( HelpTr .lt. transmissivity(idr) )      
            icomp  = icomp + 1       
            HelpTr = HelpTr + ThickComp(icomp)*CondSatHor(icomp)      
            HelpTh = HelpTh + ThickComp(icomp)       
         enddo
         NumComBotDislay(idr)    = icomp
         ThickCompBotDislay(idr) = ThickComp(icomp) - 
     &             ( HelpTr - Transmissivity(idr)) / CondSatHor(icomp)
            BotDisLay(idr)      = WatLevAv + HelpTh +
     &             ( Transmissivity(idr) - HelpTr) / CondSatHor(icomp)

C --- paragraph 8
C     correction of BotDisLay(idr) if D(idr) < 0.25 L(idr) \/(kv/kh) 
         if (BotDisLay(idr).gt. MaxDepthDislay(idr)) then
            BotDisLay(idr) = MaxDepthDislay(idr)

C --- transmissivity of order i, thickness 
C     of bottom compartment, and number 
C     of bottom compartment
            NumComBotDislay(idr) = NumComWatLev
            HelpTh               = ThickCompSatWatLev
            Transmissivity(idr)  = ThickCompSatWatLev * 
     &                             CondSatHor(NumComWatLev)
            do while ( BotDisLay(idr)-WatLevAv .gt. HelpTh )      
               NumComBotDislay(idr) = NumComBotDislay(idr) + 1       
               HelpTh = HelpTh + ThickComp(NumComBotDislay(idr))       
               Transmissivity(idr)  = Transmissivity(idr) + 
     &                               ThickComp(NumComBotDislay(idr)) * 
     &                               CondSatHor(NumComBotDislay(idr))
            enddo
            Transmissivity(idr) = Transmissivity(idr) - 
     &                            (HelpTh - BotDisLay(idr))*
     &                            CondSatHor(NumComBotDislay(idr))
            ThickCompBotDislay(idr) = ThickComp(NumComBotDislay(idr)) - 
     &                            (HelpTh - BotDisLay(idr)) 
         endif   
      enddo

C --- paragraph 9
C     Distribute drainage fluxes as lateral fluxes over 
C     ith order model discharge layers
      do iidr=1,NumActDrain
         idr = DrainSequence(iidr) 
         FluxDrComp(idr,NumComWatLev) = FluxDr(idr)*ThickCompSatWatLev*
     &                              CondSatHor(NumComWatLev) / 
     &                              Transmissivity(idr)
         do icomp=NumComWatLev+1,NumComBotDislay(idr)-1
            FluxDrComp(idr,icomp) = FluxDr(idr)*ThickComp(icomp)*
     &                              CondSatHor(icomp) / 
     &                              Transmissivity(idr)
         enddo
         FluxDrComp(idr,NumComBotDislay(idr)) = FluxDr(idr)*
     &                              ThickCompBotDislay(idr)*
     &                              CondSatHor(NumComBotDislay(idr)) / 
     &                              Transmissivity(idr)
      enddo

      return
      end


C ----------------------------------------------------------------------
      REAL*8 FUNCTION DMCNODE (NODE,HEAD,LAYER,FLGENU,COFGEN,THETLO,
     & THETHI,DMCH,DMCC,THETAS,THETAR,ALFAMG,SWMOBI,FM1,PF1,SLOPFM,
     & HTABLE,THETA)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calculate the differential moisture         
C                          capacity (as a function of pressure head)
C     Subroutines called : -                                           
C     Functions called   : thenode                                     
C     File usage         : -                                           
C     Differences SWAP/SWAPS: None
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER LAYER(MACP),NODE,THETLO(MAHO)
      INTEGER THETHI(MAHO),SWMOBI

      REAL*8  FM1(MAHO),PF1(MAHO),SLOPFM(MAHO) 

      REAL*8  HEAD,COFGEN(8,MAHO),DMCC(MAHO,99),DMCH(MAHO,99)
      REAL*8  THETAS(MACP)
      REAL*8  THETAR(MACP),ALFAMG(MACP),HTABLE(MAHO,99),THETA(MACP)

      LOGICAL FLGENU(MAHO)
C ----------------------------------------------------------------------
C --- local 

      INTEGER lay,posit

      REAL*8  fmobile

      REAL*8  deltah,head2,theta2,tresid,tsatu,thenode,alphah
      REAL*8  term1,term2
C ----------------------------------------------------------------------

C --- soil layer of node
      lay = layer(node)
      if (flgenu(lay)) then

C --- according to van Genuchten
        if (head.gt. -1.0d-8) then
          dmcnode = 1.0d-7
        else
          if (SWMOBI.eq.1) then

C --- use numerical evaluation of capacity
            deltah = 0.001 * (1.0e-3 * abs(head)+1.0d0)
            head2 = head + deltah
            if (head2.gt.0.0) then
              deltah = -deltah
              head2  = head + deltah
            endif
            if (head2.gt.-1.0d0) then
              fmobile = fm1(lay)+(0.0-pf1(lay))*slopfm(lay)
            else
              fmobile = fm1(lay)+(log10(abs(real(head2)))-pf1(lay))*
     &        slopfm(lay)
            endif
            fmobile = min (1.0d0,fmobile)
            fmobile = max (0.3d0,fmobile)
            tresid  = thetar(node)
            tsatu   = thetas(node)
            thetar(node) = cofgen(1,lay)*fmobile
            thetas(node) = cofgen(2,lay)*fmobile
            theta2 = thenode (node,head2,layer,flgenu,cofgen,
     &               htable,thetlo,alfamg,thetar,thetas)
            thetar(node) = tresid
            thetas(node) = tsatu           
            dmcnode = (theta2 - theta(node))/deltah
          else

C --- use analytical evaluation of capacity
            alphah = abs (alfamg(node)*head)

C --- compute |alpha * h| to the power n-1
            term1 = alphah ** (cofgen(6,lay) -1.0d0)

C --- compute |alpha*h| to the power n
            term2 = term1 * alphah

C --- add one and raise to the power m+1
            term2 = (1.0d0 + term2) ** (cofgen(7,lay) + 1.0d0)

C --- divide theta-s minus theta-r by term2
            term2 = (thetas(node)-thetar(node)) / term2

C --- calculate the differential moisture capacity
            dmcnode = abs(-1.0d0 * cofgen(6,lay) * cofgen(7,lay) * 
     $                alfamg(node) * term2 * term1)
          endif
          if (head.gt.-1.0d0.and.dmcnode.lt.1.0d-7) dmcnode = 1.0d-7
        endif
      else

C --- according to the tables
        if (head.ge.0.0d0) then
          dmcnode = 1.0d-7
        else
          do 100 posit = thetlo(lay)+1,thethi(lay)
            if (head.le.dmch(lay,posit)) go to 120
 100      continue
 120      dmcnode = dmcc(lay,posit-1)+(head-dmch(lay,posit-1))*
     $              (dmcc(lay,posit)-dmcc(lay,posit-1))/
     $              (dmch(lay,posit)-dmch(lay,posit-1))
        endif
      endif

      return
      end
C ----------------------------------------------------------------------
      REAL*8 FUNCTION HCONODE (NODE,WCON,LAYER,FLGENU,COFGEN,THETAR,
     & THETAS,THETLO,THETHI,KTABLE,KSE99,KSAT,SWMOBI,FMOBIL)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calculate hydraulic conductivity (as a     
C                          function of pressure head)
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C     Differences SWAP/SWAPS: Error handliong
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NODE,LAYER(MACP),THETLO(MAHO),THETHI(MAHO)
      INTEGER SWMOBI

      REAL*8  FMOBIL(MACP)
      REAL*8  WCON,COFGEN(8,MAHO),KTABLE(MAHO,99),KSAT(MAHO),KSE99(MAHO)
      REAL*8  THETAR(MACP),THETAS(MACP) 

      LOGICAL FLGENU(MAHO)

C --- Error handling
      character*80 errTxt,errSrc

C ----------------------------------------------------------------------
C --- local
      INTEGER lay,posit 

      REAL*8  term1,exponl,relsat
C ----------------------------------------------------------------------

C --- error handling (source)
      errSrc = 'HCONODE'

C --- soil layer of node
      lay = layer(node)
      if (flgenu(lay)) then

C --- calculation according to van Genuchten
        relsat = (wcon-thetar(node))/(thetas(node)-thetar(node))
        if (relsat.gt.0.99d0) then

C --- linear interpolation of k
          hconode = kse99(lay) + (relsat-0.99d0)/0.01
     &              *(ksat(lay)-kse99(lay))
        else
          if (relsat.lt.0.001d0) then
            hconode = 1.0d-10
          else
            exponl  = cofgen(5,lay)/cofgen(7,lay)-2.0d0
            term1   = (1.0d0-relsat**(1.0d0/cofgen(7,lay)))
     &                **cofgen(7,lay)
            hconode = cofgen(3,lay)*(relsat**exponl)*(1.0d0-term1)
     &               *(1.0d0-term1)
            if (SWMOBI.eq.1) hconode = hconode*fmobil(node)
          endif
        endif
      else

C ---   calculation according to the tables   
        if (wcon.lt.0.01*thetlo(lay)) then

C --- Error handling 
          errTxt = 'calculated water content outside range'
          call SetFatalError(9999,errTxt,errSrc)
          return
        else
          if (wcon.ge.(thetas(node)-0.001d0)) then
            hconode = ktable(lay,thethi(lay))
          else
            posit   = int(100.0*wcon)
            hconode = ktable(lay,posit)+(ktable(lay,posit+1)-
     &                ktable(lay,posit))*(100.0*wcon-real(posit))
          endif
        endif
      endif

      return
      end


C ----------------------------------------------------------------------
      SUBROUTINE HEADCALC (NUMNOD,HM1,H,THETM1,THETA,NUMBIT,
     & FTOPH,DT,T,KMEAN,DZ,DISNOD,DIMOCA,QROT,QDRA,POND,QTOP,
     & LAYER,FLGENU,COFGEN,HTABLE,THETLO,THETHI,ALFAMG,THETAR,THETAS,
     & DMCH,DMCC,SWBOTB,THETOL,REVA,NIRD,QBOT,QDRTOT,
     & K,KTABLE,Q,ARFLUX,KSE99,KSAT,SWMOBI,FMOBIL,VOLSAT,VOLACT,
     & DTMIN,DNOCON,QIMMOB,FM1,PF1,SLOPFM,THETIM,ADSFLU,
     & ACRACK,QCRACK,FLTSAT,HSURF,WTOPLAT,FLREVA,QROSUM,
     & QCOMP,FPEGWL,NPEGWL,GWLINP,ERRORCODE,
     & FLEVAPFRONT,FLDTMIN,REVADEEP,FLDECDTMIN,
     & THETASURF,NODEEVAPFRONT,HEVAPFRONT,HATM,EVP,FLDECDT,
     & GwlTab,GwlConv,Z)


C ----------------------------------------------------------------------
C     Date               : 15/11/01
C     Purpose            : calculate pressure heads, water contents,
C                          and conductivities for next time step
C     Subroutines called : bocotop,TriDag
C     Functions called   : hconode,dmcnode,thenode                 
C     File usage         : -
C     Notes on arguments:
C     (1) revatop input to local REVA so is NOT included in arguments 
C       (unlike revadeep).
C     (2) REVA (externally revatop) is parsed to BOCOTOP and is 
C         adjusted HERE if BOCOTOP returns FLREVA (externally flrevatop)
C         = .TRUE. 
C     (3) RevaDeep is used by SoilEvap to get the deep soil evaporation 
C         given by the array Evp
C     (4) Only one of REVA/revatop and EDed may be non-zero.
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- common global
      INTEGER NUMNOD,NUMBIT,LAYER(MACP),THETLO(MAHO)
      INTEGER THETHI(MAHO)
      INTEGER SWBOTB,DNOCON,SWMOBI,NPEGWL,NodeEvapFront,NODGWL

      REAL*8  DTMIN,DT,T,DZ(MACP),DISNOD(MACP+1),QROT(MACP)
      REAL*8  QDRA(5,MACP),POND,QTOP,QROSUM,THETOL,REVA,NIRD,QBOT
      REAL*8  QDRTOT,ARFLUX,FMOBIL(MACP),QIMMOB(MACP),FM1(MAHO)
      REAL*8  PF1(MAHO),SLOPFM(MAHO),ADSFLU(MACP),ACRACK,QCRACK,HSURF
      REAL*8  WTOPLAT,QCOMP,GWLINP
      REAL*8  HM1(MACP),H(MACP),THETM1(MACP),THETA(MACP),KMEAN(MACP+1)
      REAL*8  DIMOCA(MACP),Q(MACP+1),COFGEN(8,MAHO),HTABLE(MAHO,99)
      REAL*8  ALFAMG(MACP),THETAR(MACP),THETAS(MACP),DMCH(MAHO,99)
      REAL*8  DMCC(MAHO,99),K(MACP+1),KTABLE(MAHO,99),KSE99(MAHO)
      REAL*8  KSAT(MAHO),VOLSAT,VOLACT,THETIM(MAHO)
      REAL*8  hAtm, RevaDeep,thetaSurf,hEvapFront,Evp(macp)
      REAL*8  GwlTab(732),Z(MACP),GwlConv,Gwl,PeGwl,GwlBak(4),GwlOld

      LOGICAL FTOPH,FLGENU(MAHO),FLTSAT,FLREVA,FPEGWL
      LOGICAL FlDecDtMin,FlDtMin,FlEvapFront,FlDecDt

C ----------------------------------------------------------------------
C --- local
      INTEGER ErrorCode,i,j,lay,GetErrorStatus

      REAL*8  qimold(MACP),fmobm1(MACP),deno,qthetaEF
      REAL*8  thenode,hconode,thoma(MACP),thomb(MACP),thomc(MACP)
      REAL*8  thomf(MACP),capold(MACP),dmcnode,hold(MACP),theold(MACP)

      LOGICAL      FLAdjDT
      CHARACTER*80 ErrTxt,ErrSrc    
	EXTERNAL     GetErrorStatus

      save
C ----------------------------------------------------------------------

C --- error handling: name of potential error
      ErrSrc = 'HeadCalc'

C --- store values of h, theta and fmobil
      do i = 1,numnod
        hm1(i)    = h(i)
        thetm1(i) = theta(i)
        fmobm1(i) = fmobil(i)
      enddo

      if (FlEvapFront) then
C ---   use SoilEvapFront, option in SWAPS
        call SoilEvap(NumNod,NodeEvapFront,dt,dz,hEvapFront,
     &    REvaDeep,theta,qRot,LAYER,FLGENU,COFGEN,HTABLE,
     &    THETLO,ALFAMG,THETAR,THETAS,Evp,qthetaEF)
      endif

      numbit = 1

C --- first step:
C --- ===========
C --- calculation of coefficients for node = 1
      i = 1
      if (ftoph) then

C ---   h at soil surface prescribed
        thomc(i) = - dt*kmean(i+1)/dz(i)/disnod(i+1)
        thomb(i) = - thomc(i)+dimoca(i)+dt*kmean(1)/disnod(i)/dz(i)
        thomf(i) = dimoca(i)*h(i)-qimmob(i) +
     &             dt/dz(i)*( +kmean(i)-kmean(i+1)-qrot(i)+adsflu(i)
     &             - qdra(1,i) - qdra(2,i) - qdra(3,i) - qdra(4,i)
     &             - qdra(5,i) - Evp(i) )
     &             + dt*kmean(i)*hsurf/disnod(i)/dz(i)
      else

C ---   q at soil surface prescribed
        thomc(i) = - dt*kmean(i+1)/dz(i)/disnod(i+1)
        thomb(i) = - thomc(i) + dimoca(i)
        thomf(i) = dimoca(i)*h(i) -qimmob(i)+
     &             dt/dz(i)*( -qtop-kmean(i+1)-qrot(i)+adsflu(i)
     &             - qdra(1,i) - qdra(2,i) - qdra(3,i) - qdra(4,i)
     &             - qdra(5,i) - Evp(i) )
      endif

C --- calculation of coefficients for 2 < node < numnod
      do i = 2, numnod-1
        thoma(i) = - dt*kmean(i)/dz(i)/disnod(i)
        thomc(i) = - dt*kmean(i+1)/dz(i)/disnod(i+1)
        thomb(i) = - thoma(i) - thomc(i) + dimoca(i)
        thomf(i) = dimoca(i)*h(i) -qimmob(i)+
     &             dt/dz(i)*(kmean(i)-kmean(i+1)-qrot(i)+adsflu(i)
     &             - qdra(1,i) - qdra(2,i) - qdra(3,i) - qdra(4,i)
     &             - qdra(5,i) - Evp(i) )
      enddo

C --- calculation of coefficients for node = numnod
      i = numnod
      if (swbotb .eq. 1 .or. swbotb .eq. 5) then
        thoma(i) = 0.0
        thomb(i) = 1.0d0
        thomf(i) = h(i)
      else
        thoma(i) = - dt*kmean(i)/dz(i)/disnod(i)
        thomb(i) = - thoma(i) + dimoca(i)
        thomf(i) = dimoca(i)*h(i) -qimmob(i)+
     &             dt/dz(i)*(kmean(i)+qbot-qrot(i)+adsflu(i)
     &             - qdra(1,i) - qdra(2,i) - qdra(3,i) - qdra(4,i)
     &             - qdra(5,i) - Evp(i) )
      endif

C     save old values of h, Theta and DiMoCa
      call CALCGWL(GWL,GWLBAK,NUMNOD,H,Z,DZ,SWBOTB,POND,NODGWL,
     &           FPEGWL,NPEGWL,PEGWL,GWLTAB,T)

      gwlold = gwl
      do i = 1, NumNod
        hOld(i)   = h(i)
        TheOld(i) = Theta(i)
        CapOld(i) = DiMoCa(i)
      enddo

C --- solve the tridiagonal matrix
      call TriDag(NumNod, thoma, thomb, thomc, thomf, h,ErrorCode)

C --- return if numerical solution fails
      if (ErrorCode .ne. 0) return

C --- first solution
      do i = 1,numnod
        lay = layer(i)
        if (SWMOBI.eq.1) then
          if (h(i).gt.-1.0d0) then
            fmobil(i) = fm1(lay)+(0.0-pf1(lay))*slopfm(lay)
          else
            fmobil(i) = fm1(lay)+(log10(abs(real(h(i))))-pf1(lay))*
     &                  slopfm(lay)
          endif
          fmobil(i) = min (1.0d0,fmobil(i))
          fmobil(i) = max (0.3d0,fmobil(i))
          thetar(i) = cofgen(1,lay)*fmobil(i)
          thetas(i) = cofgen(2,lay)*fmobil(i)
          qimold(i) = qimmob(i)
          qimmob(i) = (fmobm1(i)-fmobil(i))*thetim(lay)
        endif
        theta(i)  = thenode(i,h(i),LAYER,FLGENU,COFGEN,HTABLE,
     &              THETLO,ALFAMG,THETAR,THETAS)
        dimoca(i) = dmcnode(i,h(i),LAYER,FLGENU,COFGEN,THETLO,THETHI,
     &              DMCH,DMCC,THETAS,THETAR,ALFAMG,SWMOBI,FM1,PF1,
     &              SLOPFM,HTABLE,THETA)
      enddo

C --- iteration step : 
C --- ================
 100  continue

      if (FlEvapFront) then
C ---   use SoilEvapFront, option in SWAPS
        call SoilEvap(NumNod,NodeEvapFront,dt,dz,hEvapFront,
     &       REvaDeep,theta,qRot,LAYER,FLGENU,COFGEN,HTABLE,
     &       THETLO,ALFAMG,THETAR,THETAS,Evp,qthetaEF)
      endif

C --- updating coefficients tridiagonal Thomas matrix
      i = 1
      if (ftoph) then

C ---   head controlled boundary 
        thomb(i) = - thomc(i)+dimoca(i)+dt*kmean(1)/disnod(i)/dz(i)
        thomf(i) = -theta(i)+thetm1(i)-qimmob(i)+qimold(i)+
     &             dimoca(i) * h(i) +
     &             dt/dz(i)*( +kmean(i)-kmean(i+1)-qrot(i)+adsflu(i)
     &             - qdra(1,i) - qdra(2,i) - qdra(3,i) - qdra(4,i)
     &             - qdra(5,i) - Evp(i) ) 
     &             + dt*kmean(i)*hsurf/disnod(i)/dz(i)
      else

C ---   flux controlled boundary
        thomb(i) = - thomc(i) + dimoca(i)
        thomf(i) = -theta(i) + thetm1(i)-qimmob(i)+qimold(i)+
     &             dimoca(i) * h(i) +
     &             dt/dz(i)*( -qtop-kmean(i+1)-qrot(i)+adsflu(i)
     &             - qdra(1,i) - qdra(2,i) - qdra(3,i) - qdra(4,i)
     &             - qdra(5,i) - Evp(i) )
      endif
          
      do i = 2,numnod
        thomb(i) = thomb(i) - capold(i) + dimoca(i)
        thomf(i) = thomf(i) - capold(i) * hold(i) + dimoca(i) * h(i)
     &             - theta(i) + theold(i)-qimmob(i)+qimold(i)
      enddo

C --- update qbot in case of free drainage
      if (swbotb.eq.7) then 
        i = numnod
        k(i) = hconode(i,theta(i),LAYER,FLGENU,COFGEN,
     &         THETAR,THETAS,THETLO,THETHI,KTABLE,
     &         KSE99,KSAT,SWMOBI,FMOBIL)
	  if (GetErrorStatus().ne.0) return
	  qbot = -1.0d0 * k(i)
        thomf(i) = dimoca(i)*h(i) -qimmob(i)+
     &             dt/dz(i)*(kmean(i)+qbot-qrot(i)+adsflu(i)
     &             -qdra(1,i)-qdra(2,i)-qdra(3,i)-qdra(4,i)-qdra(5,i))
      endif

      i = numnod
      if (swbotb .eq. 1 .or. swbotb .eq. 5) then
        thomb(i) = 1.0d0
        thomf(i) = h(i)
      endif

      if (swbotb .eq. 8 .and. h(i) .gt. (-disnod(i+1)+0.001d0)) then
        thoma(i) = 0.0d0
        thomb(i) = 1.0d0
        thomf(i) = - disnod(i+1)
      endif

C --- save old values of h, Theta and DiMoCa
      gwlold = gwl
      do i = 1, NumNod
        hOld(i)   = h(i)
        TheOld(i) = Theta(i)
        CapOld(i) = DiMoCa(i)
      enddo

C --- solve the tridiagonal matrix.
      call TriDag(NumNod, thoma, thomb, thomc, thomf, h,ErrorCode)

      call CALCGWL(GWL,GWLBAK,NUMNOD,H,Z,DZ,SWBOTB,POND,NODGWL,
     &           FPEGWL,NPEGWL,PEGWL,GWLTAB,T)


C --- return if numerical solution fails
      if (ErrorCode .ne. 0) return

C --- first solution
      do i = 1,numnod
        lay = layer(i)
        if (SWMOBI.eq.1) then
          if (h(i).gt.-1.0d0) then
            fmobil(i) = fm1(lay)+(0.0-pf1(lay))*slopfm(lay)
          else
            fmobil(i) = fm1(lay)+(log10(abs(real(h(i))))-pf1(lay))*
     &                  slopfm(lay)
          endif
          fmobil(i) = min (1.0d0,fmobil(i))
          fmobil(i) = max (0.3d0,fmobil(i))
          thetar(i) = cofgen(1,lay)*fmobil(i)
          thetas(i) = cofgen(2,lay)*fmobil(i)
          qimold(i) = qimmob(i)
          qimmob(i) = (fmobm1(i)-fmobil(i))*thetim(lay)
        endif
        theta(i) = thenode(i,h(i),LAYER,FLGENU,COFGEN,HTABLE,
     &             THETLO,ALFAMG,THETAR,THETAS)
        dimoca(i) = dmcnode(i,h(i),LAYER,FLGENU,COFGEN,THETLO,
     &              THETHI,DMCH,DMCC,THETAS,THETAR,ALFAMG,SWMOBI,
     &              FM1,PF1,SLOPFM,HTABLE,THETA)
      enddo

C --- check on convergence of solution
      do i = 1,numnod
        if (abs(h(i)) .lt. 10.0d0) then
          deno = 10.0d0
        else
          deno = h(i)
        endif
        if (abs(theold(i)-theta(i)) .gt. thetol
     &    .or. abs( (h(i)-hold(i)) / deno ) .gt. 0.01d0
     &    .or. h(1) .gt. 50.0d0 .or. abs(Gwl-GwlOld).ge.GwlConv) then

          if (numbit .lt. 6) then

C ---       perform another iteration
            numbit = numbit + 1
            call BOCOTOP(POND,REVA,NIRD,DT,QTOP,QBOT,QDRTOT,
     &      KMEAN,K,H,DISNOD,LAYER,FLGENU,COFGEN,THETA,THETIM,DZ,
     &      HTABLE,THETLO,THETHI,KTABLE,ALFAMG,FTOPH,ARFLUX,THETAR,
     &      THETAS,KSE99,KSAT,SWMOBI,FMOBIL,VOLSAT,VOLACT,ACRACK,QCRACK,
     &      FLTSAT,HSURF,DTMIN,WTOPLAT,FLREVA,NUMNOD,QROSUM,QCOMP,
     &      FPEGWL,NPEGWL,Q,SWBOTB,GWLINP,hAtm,FlDecDtMin,revadeep)
            if (GetErrorStatus().ne.0) return
            goto 100

          else
            if (.not.FlDtMin) then

C ---         reset and continue iteration with smaller timestep!
              FlDecDt = .true.

C ---         reset soil state variables
              if (SWMOBI.eq.1) then
                qimmob(1) = 0.0
                fmobil(1) = fmobm1(1)
                thetar(1) = cofgen(1,1)*fmobil(1)
                thetas(1) = cofgen(2,1)*fmobil(1)
              endif
              h(1) = hm1(1)
              theta(1) = thetm1(1)
              k(1) = hconode(1,theta(1),LAYER,FLGENU,COFGEN,
     &               THETAR,THETAS,THETLO,THETHI,KTABLE,
     &               KSE99,KSAT,SWMOBI,FMOBIL)
              dimoca(1) = dmcnode(1,hm1(1),LAYER,FLGENU,COFGEN,
     &               THETLO,THETHI,DMCH,DMCC,THETAS,THETAR,ALFAMG,
     &               SWMOBI,FM1,PF1,SLOPFM,HTABLE,THETA)
              do j = 2,numnod
                lay = layer(j)
                if (SWMOBI.eq.1) then
                  qimmob(j) = 0.0
                  fmobil(j) = fmobm1(j)
                  thetar(j) = cofgen(1,lay)*fmobil(j)
                  thetas(j) = cofgen(2,lay)*fmobil(j)
                endif
                h(j) = hm1(j)
                theta(j) = thetm1(j)
                k(j) = hconode(j,theta(j),LAYER,FLGENU,
     &                 COFGEN,THETAR,THETAS,THETLO,THETHI,KTABLE,
     &                 KSE99,KSAT,SWMOBI,FMOBIL)
                dimoca(j) = dmcnode(j,h(j),LAYER,FLGENU,COFGEN,THETLO,
     &                 THETHI,DMCH,DMCC,THETAS,THETAR,ALFAMG,SWMOBI,
     &                 FM1,PF1,SLOPFM,HTABLE,THETA)
                kmean(j)  = 0.5*(k(j) + k(j-1))
              enddo
              kmean(numnod+1) = k(numnod)

              if (GetErrorStatus().ne.0) return

C ---         decrease time step in TIMER
              return  

            else
C ---         continue without convergence 

              if (dnocon .eq. 0) then 
C ---           set warning at first day without convergence
	          dnocon = INT(T) 
                errTxt='No convergence of soil moisture calculations at'
                call SetWarning(5010,ErrTxt,ErrSrc)
              endif  
            endif

          endif
        endif
      enddo

C --- flux at soil surface according to Darcy.
      if (ftoph) qtop = -kmean(1)*((hsurf - h(1))/disnod(1)+1.0d0)

C --- adjust conductivities
      k(1) = hconode(1,theta(1),LAYER,FLGENU,COFGEN,THETAR,THETAS,
     &  THETLO,THETHI,KTABLE,KSE99,KSAT,SWMOBI,FMOBIL)
      do i = 2,numnod
        k(i) = hconode(i,theta(i),LAYER,FLGENU,COFGEN,THETAR,
     &         THETAS,THETLO,THETHI,KTABLE,KSE99,KSAT,SWMOBI,
     &         FMOBIL)
        kmean(i) = 0.5*(k(i)+k(i-1))
	  if(GetErrorStatus().ne.0) return
      enddo
      kmean(numnod+1) = k(numnod)

C --- reva in case of SWAPS; reva of SWAP is recalculated in FLUXES
      if (flreva) then
        reva = qtop + (pond-wtoplat)/dt + (arflux+nird)*(1.0d0-acrack)
	endif

C --- conceptual theta at surface calculated here for output only
      thetaSurf  = thenode(1,hAtm,LAYER,FLGENU,COFGEN,HTABLE,
     &             THETLO,ALFAMG,THETAR,THETAS)
      REvaDeep = qthetaEF

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE OUTDRF (LOGF,OUTNAM,DRF,HDRFD,ENVSTR,DAYNR,YEAR,
     &  NRPRI,NRSRF,CQDRAIN,CQDRD,CRUNO,CQDRAR,T)
C ----------------------------------------------------------------------
C     Date               : 10/6/99
C     Purpose            : write drainage fluxes, surface runoff, rapid
C                          drainage to  OUTNAM.DRF file
C     Subroutines called : 
C     Functions called   : -
C     File usage         : OUTNAM.DRF
C     Differences SWAP/SWAPS: 
C     - Extra SWAPS integer in file handling
C     - Output format (for GUI SWAP)
C ---------------------------------------------------------------------
      IMPLICIT NONE

C --- Global
      INTEGER   DRF,DAYNR,YEAR,D,M,NRPRI,NRSRF, logf
      REAL*8    CQDRAIN(5),CQDRD,CRUNO,CQDRAR,T
      CHARACTER OUTNAM*16,FILNAM*80,ENVSTR*50

C --- Local
      INTEGER   LEVEL
      integer*4 getun
      REAL*8    C1QDRAIN(5),C1QDRD,C1RUNO,C1QDRAR
      LOGICAL   HDRFD

      save
C ----------------------------------------------------------------------
      if (.NOT.HDRFD) then

C --- open output file
        filnam = trim(envstr)//trim(outnam)//'.drf'
        drf = getun (20,30)
        call rdinit(drf,logf,filnam)

C --- write header
        if (nrpri .eq. 1) then
          write (DRF,100) NRSRF
        else
          write (DRF,101) NRSRF
        endif

C --- Output format
 100    format (
     &'* CUMULATIVE DRAINAGE FLUXES, SURFACE RUNOFF AND RAPID',
     &' DRAINAGE:',/,'*',/,
     &'* Total number of levels for drainage fluxes :',I3,/,'*',/
     &'* First level (primary system) NOT included in sw-reservoir',/,
     &'*',/)

 101    format (
     &'* CUMULATIVE DRAINAGE FLUXES, SURFACE RUNOFF AND RAPID',
     &' DRAINAGE:',/,'*',/,
     &'* Total number of levels for drainage fluxes :',I3,/,'*',/
     &'* First level (primary system) IS included in sw-reservoir',/,
     &'*')

        write (DRF,11)
 11     format (
     &'* Meaning of symbols:',/,
     &'* - CQD(*)  : Drainage fluxes all levels',/,
     &'* - CQDRD   : Total of drainage fluxes into surface',
     &            ' water reservoir (SWSRF>=1)',/,
     &'* - CRUNO   : Surface runoff (increment can be <0.0)',/,
     &'* - CQDRAR  : Rapid drainage (increment always >= 0.0)',/,
     &'* - cum     : cumulative value',/,
     &'* - inc     : incremental value',/,'*',/, 
     &'* DATE            CQD(1)  CQD(2)  CQD(3)  CQD(4)  CQD(5)',
     &'   CQDRD   CRUNO  CQDRAR',/,
     &'* dd/mm/yyyy  nr    inc/    inc/    inc/    inc/    inc/',
     &'    inc/    inc/    inc/',/,
     &'*                    cum     cum     cum     cum     cum',
     &'     cum     cum     cum',/,
     &'*                   [cm]    [cm]    [cm]    [cm]    [cm]',
     &'    [cm]    [cm]    [cm]',/,
     &'* <============><======><======><======><======><======>',
     &'<======><======><======>')

        do 30 level=1,5
          c1qdrain(level)=0.0
 30     continue
        c1qdrd   = 0.0
        c1runo   = 0.0
        c1qdrar  = 0.0
        hdrfd    = .true.
      endif

C --- conversion of daynumber to date
      call NRTODA (YEAR,DAYNR,M,D)

C --- write output record
      write (DRF,20) D,M,YEAR,NINT(T),((cqdrain(level)-c1qdrain(level)),
     &  level=1,5),(cqdrd-c1qdrd),(cruno-c1runo),(cqdrar-c1qdrar),
     &  D,M,YEAR,NINT(T),(cqdrain(level),level=1,5),cqdrd,cruno,cqdrar

 20   format ('1',1X,I2,'/',I2.2,'/',2I4,8f8.2,/,
     &        '2',1X,I2,'/',I2.2,'/',2I4,8f8.1)

C --- store cumulative values
      do 50 level=1,5
          c1qdrain(level)=cqdrain(level)
 50   continue
      c1qdrd   = cqdrd
      c1runo   = cruno
      c1qdrar  = cqdrar

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE OUTSWB (LOGF,OUTNAM,SWB,HSWBD,ENVSTR,DAYNR,YEAR,
     & GWL,POND,wlstar,wls,WSTINI,wst,CQDRD,CRUNO,CQDRAR,CWSUPP,CWOUT,
     & NUMADJ,HWLMAN,VTAIR,OVERFL,SWSEC,SWMAN,NMPER,IMPEND,IMPER,
     & HBWEIR,T)
C ----------------------------------------------------------------------
C     Date               : 21/08/99
C     Purpose            : write surface water balance data to 
C                          OUTNAM.SWB file, surface water management
C                          data to OUTNAM.MAN. The files overlap
C     Subroutines called : 
C     Functions called   : -
C     File usage         : OUTNAM.SWB, OUTNAM.MAN
C ---------------------------------------------------------------------

      IMPLICIT NONE

C --- Global
      INTEGER   SWB,MAN,DAYNR,YEAR,D,M,swsec,swman(10),nmper,impend(10)
      INTEGER   imper,numadj, logf
      REAL*8    GWL,POND,WLSTAR,WSTINI,WLS,WST,CQDRD,CRUNO
      REAL*8    CQDRAR,CWSUPP,CWOUT,hwlman,vtair,hbweir(nmper),t
      CHARACTER OUTNAM*16,FILNAM*80,ENVSTR*50
      LOGICAL   OVERFL

C --- Local
      REAL*8    GWLEV,CQDRF1,C1WSUPP,C1WOUT,DELBAL
      CHARACTER SPC*1
      integer*4 getun
      LOGICAL   HSWBD

	character*80 errTxt,errSrc

      save
C ----------------------------------------------------------------------
	errSrc= 'OUTSWB' 

C ---   determine which management period the model is in:
      if (SWSEC .eq. 2) then
        imper   = 0
   5    imper   = imper + 1
C       Error handling
        if (imper .gt. nmper) then
	    errTxt = 'error sw-management periods'
		call SetFatalError(9999,errTxt,errSrc)
	    return
        endif
	  if (int(T) .gt. impend(imper)) goto 5
      endif

C --- add ponding to groundwater level, if gwl at soil surface:
      if (abs(gwl) .lt. 1.0d-7) then
          gwlev = pond
      else
          gwlev = gwl
      endif
      if (.NOT.HSWBD) then

C --- open output file
        filnam = trim(envstr)//trim(outnam)//'.swb'
        swb = getun (20,30)
        call rdinit(swb,logf,filnam)

C --- write header
        write (SWB,10)
        write (SWB,11)

 10     format (
     &'* Surface water system:',/,'*',/,
     &'* - GWL           : groundwater level + ponding (<0. = below ',
     &'s.s)',/,
     &'* - WLST          : target level,  aut. weir (<0. = b.s.s.) ',/,
     &'* - WLST          : weir crest,   fixed weir (<0. = b.s.s.) ',/,
     &'* - WLS           : surface water level (<0. = b.s.s.) ',/,
     &'* - WST           : storage in sw-reservoir, per unit area of',
     &'* subcatchment',/,
     &'* - D+RO+R        : drainage fluxes (>0. = into sw) + runoff +',
     &'* rapid drainage')

 11     format (
     &'* - QSUPP         : external supply to sw-reservoir',/, 
     &'* - QOUT          : outflow from sw-reservoir',/,'*',/,
     &'*DATE              GWL   WLST    WLS   WST D+RO+R  QSUPP',
     &'   QOUT D+RO+R  QSUPP   QOUT',/,
     &'*dd/mm/yyyy  nr                               inc    inc',
     &'    inc    cum    cum    cum',/,
     &'*                 [cm]   [cm]   [cm]  [cm]   [cm]   [cm]',
     &'   [cm]   [cm]   [cm]   [cm]',/,
     &'*<============><=====><=====><=====><====><=====><=====>',
     &'<=====><=====><=====><=====>')

C --- write record for initial state
        write (SWB,19) gwlev,wlstar,wls,wst,(0.),(0.),(0.),(0.),
     &   (0.),(0.)

 19     format ('*>> initial    ',3f7.1,f6.1,6f7.2)

C --- compose filename for management data
        if (SWSEC .eq. 2) then

C --      open output file
          filnam = trim(envstr)//trim(outnam)//'.man'
          man = getun (20,30)
          call rdinit(man,logf,filnam)

C --      write 
          write (MAN,20)
          write (MAN,21)
 20       format (
     &'* Surface water management:',/,'*',/,
     &'* - SWMAN         : type of weir (f = fixed; a = automatic)',/
     &'* - GWL           : groundwater level + ponding (<0. = below ',
     &'s.s)',/,
     &'* - HWLMAN        : pressure head used for target level (cm)',/,
     &'* - VTAIR         : total air volume in soil profile (cm)',/
     &'* - WLST          : target level of autom. weir (<0. = b.s.s.) '
     & ,/,
     &'*              or : crest  level of fixed  weir (<0. = b.s.s.) '
     & ,/,
     &'* - WLS           : surface water level (<0. = b.s.s.) ',/,
     &'* - QOUT          : surface water outflow (<0. = supply)',/,
     &'* - NUMADJ        : number of adjustments of target level',/,
     &'* - OVERFL        : flag for overflow of aut. weir (o)',/,
     &'* - CREST         : Crest level (<0. = b.s.s.)',/,'*')
 21       format (
     &'*DATE           SWMAN     GWL HWLMAN  VTAIR   WLST    WLS   QOUT'
     &,' NUMADJ OVERFL  CREST',/,
     &'*dd/mm/yyyy  nr          [cm]   [cm]   [cm]   [cm]   [cm]   [cm]'
     &'   [cm]',/,
     &'*<============><=====><=====><=====><=====><=====><=====><=====>'
     &,'<=====><=====><=====>')

        endif
        cqdrf1  = 0
        c1wsupp = 0
        c1wout  = 0
        HSWBD   = .true.
        return
      endif

C --- water balance error (water creation)
      delbal = (wst + cwout) - (wstini + cqdrd+cruno+cqdrar + cwsupp)
      if (delbal .gt. 0.05d0) then
         write(6,1001) delbal
 1001    format(/' >>Error in cumulative water balance of sw:',f5.2)
      endif

C --- write output record OUTNAM.SWB
C --- conversion of daynumber to date
      call NRTODA (YEAR,DAYNR,M,D)
      write (SWB,30) D,M,YEAR,NINT(T),gwlev,wlstar,wls,wst,
     &  (cqdrd+cruno+cqdrar-cqdrf1),(cwsupp-c1wsupp),(cwout-c1wout),
     &  (cqdrd+cruno+cqdrar),cwsupp,cwout
 30   format (1x,I2,'/',I2.2,'/',2I4,3f7.1,f6.1,6f7.2)

C --- write output record OUTNAM.MAN
      if (swsec .eq. 2) then
        if (overfl) then
          spc = 'o'
        else
          spc = '-'
        endif
        if (SWMAN(imper) .eq. 1) then
          write (MAN,31) D,M,YEAR,NINT(T),gwlev,wlstar,wls,
     &    ((cwout-c1wout)-(cwsupp-c1wsupp)),numadj,spc,hbweir(imper)
        else
          write (MAN,32) D,M,YEAR,NINT(T),gwlev,hwlman,vtair,wlstar,wls,
     &    ((cwout-c1wout)-(cwsupp-c1wsupp)),numadj,spc,hbweir(imper)
        endif
 31     format (1x,I2,'/',I2.2,'/',2i4,4x,'f',2x,f7.1,2('     - '),
     &          3f7.1,i7,6x,a1,f7.1)
 32     format (1x,I2,'/',I2.2,'/',2i4,4x,'a',2x,6f7.1,i7,6x,a1,f7.1)
      endif

C --- store cumulative values
      cqdrf1  = (cqdrd+cruno+cqdrar)
      c1wsupp = cwsupp
      c1wout  = cwout

      return
      end

C ----------------------------------------------------------------------
      REAL*8 FUNCTION PRHNODE (NODE,WCON,LAYER,DISNOD,H,FLGENU,
     &                THETAR,THETAS,COFGEN,ALFAMG,HTABLE)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calculate pressure head at nodal      
C                          point node from water content wcon
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C     Differences SWAP/SWAPS: None
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NODE,LAYER(MACP)

      REAL*8  DISNOD(MACP+1)
      REAL*8  WCON,H(MACP),COFGEN(8,MAHO),HTABLE(MAHO,99),ALFAMG(MACP)
      REAL*8  THETAR(MACP),THETAS(MACP) 

      LOGICAL FLGENU(MAHO)
C ----------------------------------------------------------------------
C --- local
      INTEGER posit,lay

      REAL*8  help
C ----------------------------------------------------------------------

C --- soil layer of node
      lay = layer(node)
      if (thetas(node)-wcon.lt.1.0d-6) then

C --- saturated pressure head
        if (node.eq.1) then
          prhnode = disnod(node)
        else
          prhnode = h(node-1) + disnod(node)
        endif
      else
        if (flgenu(lay)) then

C --- pressure head according to van Genuchten
          if (wcon-thetar(node).lt.1.0d-6) then
            prhnode = -1.0d12
          else

C --- first calculate the inverse of the sorptivity
            help = (thetas(node) - thetar(node)) / 
     $             (wcon - thetar(node))
C --- raise to the power 1/m
            help = help ** (1.0d0 / cofgen(7,lay))
C --- subtract one and raise to the power 1/n 
            help = (help - 1.0d0) ** (1.0d0 / cofgen(6,lay))
C ---       divide by alpha.
            prhnode = -1.0d0 * abs(help/alfamg(node))
          endif
        else
C --- interpolated pressure head from the table
          posit   = int(max (1.0,real(wcon*100.0)))
          prhnode = htable(lay,posit) + (htable(lay,posit+1)-
     $              htable(lay,posit))*(wcon-posit*0.01d0) / 0.01d0
        endif
      endif

      return
      end

C ----------------------------------------------------------------------
      real*8 function QHTAB (wlev,imper,hqhtab,qqhtab)
C-----------------------------------------------------------------------
C     Date               : 23/08/99
C     Purpose            : calculate surface water discharge from  
C                          surface water level, using table
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C     Verschillen Swaps en Swap: geen
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER imper,itab

      REAL*8  wlev, hqhtab(10,10),qqhtab(10,10),dwl
C ----------------------------------------------------------------------

      ITAB = 2
 100  if (wlev.lt.HQHTAB(IMPER,ITAB)) then 
        ITAB = ITAB+1
        goto 100
      endif
      dwl  = (wlev-HQHTAB(IMPER,ITAB))/
     &       (HQHTAB(IMPER,ITAB-1)-HQHTAB(IMPER,ITAB))
      qhtab = QQHTAB(IMPER,ITAB) + dwl *
     &        (QQHTAB(IMPER,ITAB-1)-QQHTAB(IMPER,ITAB))  

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE SLTBAL (DT,NIRD,CIRR,ARFLUX,CPRE,QBOT,CDRAIN,CML,
     &     NUMNOD,QDRTOT,RAPDRA,CRACKC,SQBOT,SQSUR,SQRAP,SQPREC,SQIRRIG)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calculation of solute balance           
C     Subroutines called : -                                           
C     Functions called   : -                                           
C     File usage         : -
C     Differences SWAP/SWAPS: None
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NUMNOD

      REAL*8  DT,NIRD,CIRR,ARFLUX,CPRE,QBOT,CDRAIN,CML(MACP),QDRTOT
      REAL*8  RAPDRA,CRACKC,SQBOT,SQSUR,SQRAP,SQPREC,SQIRRIG
C ----------------------------------------------------------------------

C --- add time step fluxes to total cumulative values
      sqprec  = sqprec + arflux*cpre*dt
      sqirrig = sqirrig + nird*cirr*dt

      if (qbot .gt. 0.0) then
        sqbot = sqbot + qbot*cdrain*dt
      else 
        sqbot = sqbot + qbot*cml(numnod)*dt
      endif
      sqrap = sqrap + rapdra*crackc*dt
      sqsur = sqsur + qdrtot*cdrain*dt

      return
      end


C ----------------------------------------------------------------------
      SUBROUTINE SOLUTE (CPRE,NIRD,CIRR,DT,NUMNOD,LAYER,
     & INPOLA,INPOLB,CML,THETA,DDIF,LDIS,Q,DISNOD,THETSL,
     & Z,GAMPAR,RTHETA,BEXP,DECPOT,FDEPTH,KF,FREXP,FMOBIL,
     & TSCF,QROT,DZ,KMOBIL,CIL,QDRA,CMSY,THETIM,CISY,CL,DTSOLU,
     & DTMAX,TSoil,CPOND,QTOP,WTOPLAT,ACRACK,ADSFLU,CRACKC,
     & ARFLUX,SWCRACK,CRACKW,CKWM1,RAPDRA,QCRACK,POND,CREF,QDRTOT,
     & POROS,KFSAT,DECSAT,CDRAIN,HAQUIF,DECTOT,SAMPRO,SAMCRA,SAMAQ,
     & CRALEV,DIFDES,ROTTOT,SQDRA,SWBR,SDRAIN,BDENS,SWHEA,CSURF)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calculation of solute concentrations
C     Subroutines called : -                                           
C     Functions called   : -                                           
C     File usage         : -                                           
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER NUMNOD,LAYER(MACP),SWCRACK,SWBR,SWHEA

      REAL*8  CPRE,NIRD,CIRR,DT,INPOLA(MACP),INPOLB(MACP),CML(MACP)
      REAL*8  DDIF,LDIS,DISNOD(MACP+1),Z(MACP)
      REAL*8  GAMPAR,RTHETA,BEXP,DECPOT,FDEPTH(MAHO),KF,FREXP
      REAL*8  FMOBIL(MACP),TSCF,QROT(MACP),DZ(MACP),KMOBIL,CIL(MACP)
      REAL*8  QDRA(5,MACP),ARFLUX,BDENS(MAHO)
      REAL*8  CMSY(MACP),CISY(MACP),CL(MACP),DTSOLU,DTMAX,CREF
      REAL*8  CPOND,QTOP,WTOPLAT,ACRACK,ADSFLU(MACP),CRACKC
      REAL*8  CRACKW,CKWM1,RAPDRA,QCRACK,POND,QDRTOT,POROS,KFSAT
      REAL*8  DECSAT,CDRAIN,HAQUIF,DECTOT,CRALEV,DIFDES,ROTTOT
      REAL*8  SAMAQ,SAMCRA,SAMPRO,SQDRA,SDRAIN,CSURF
      REAL*8  THETA(MACP),Q(MACP+1),THETIM(MAHO)
      REAL*8  THETSL(MAHO)
      REAL*8  TSoil(0:MACP)

      LOGICAL differ

C ----------------------------------------------------------------------
C --- local
      INTEGER level,lay,i 

      REAL*8  cmlav,thetav,ftemp,ftheta,decact,cfluxt,cfluxb
      REAL*8  cdrtot,ctrans,cdiff,crot,dispr,diffus,old,dummy,rer
      REAL*8  in,vpore,desflu(MACP+1),deswat,desconc,destot,isqdra
      REAL*8  samount
C ----------------------------------------------------------------------
      parameter (rer = 0.001d0)

      save
C ----------------------------------------------------------------------
      isqdra  = 0.0
      samount = 0.0

C --- flux at surface
      csurf = (NIRD*cirr + ARFLUX*cpre)*dt + csurf
      if (qtop.lt.-1.d-6) then

C --- For SWAP
        cpond  = csurf / (pond-qtop*dt+wtoplat)
        cfluxt = qtop*(1.0d0-acrack)*cpond*dt
	  csurf  = csurf + cfluxt - wtoplat*cpond
      else
        cpond  = 0.0
        cfluxt = 0.0
      endif

C --- solute balance in cracks
      if (SWCRACK.eq.1) then
        in = wtoplat*cpond + ARFLUX*acrack*cpre*dt + destot
        if (crackw.gt.1.0d-5) then
          crackc = (crackc*ckwm1 + in)/(crackw+(rapdra+qcrack)*dt)
        else
          crackc = 0.0
        endif
      endif

C --- Flux and concentration of water flowing through clay cracks
      if (SWCRACK.eq.1) then
        destot = 0.0
        deswat = wtoplat + arflux*acrack*dt
        if (deswat .gt. 1.d-7) then
          desconc = (wtoplat*cpond + arflux*acrack*cpre*dt) / deswat
        else
          desconc = 0.0
        endif
      endif

C --- Start loop of calculations per compartment
      do  500 i = 1,numnod
        lay = layer(i)

C --- Mobile phase -----------------------------------------------------

C --- mass exchanged between compartments
        if (i .lt. numnod) then
          cmlav  = inpola(i+1) * cml(i) + inpolb(i) * cml(i+1)
          thetav = inpola(i+1)*theta(i)+inpolb(i)*theta(i+1)
          vpore  = abs(q(i+1))/thetav
          diffus = ddif*(thetav**2.33)/(thetsl(lay)*thetsl(lay))
          dispr  = diffus + ldis * vpore + 0.5 * dt * vpore * vpore
          cfluxb = (q(i+1)*cmlav +thetav * dispr 
     &               *(cml(i+1)-cml(i))/disnod(i+1))*dt
        else
          if (q(i+1).gt.0.0) then
            cfluxb = q(i+1)*cdrain*dt
          else
            cfluxb = q(i+1)*cml(i)*dt
          endif
        endif

C --- transformation of the substance
        if (swhea .eq. 1) then
	    if (TSoil(i) .lt. 35.0d0) then
            ftemp = exp(gampar*(TSoil(i)-20.0d0))
	    else
            ftemp = exp(gampar*15.0)
          endif
	  else
	    ftemp = 1.0d0
	  endif
        ftheta = min(1.0d0,(theta(i)/rtheta)**bexp)
        decact = decpot*ftemp*ftheta*fdepth(lay)
        ctrans = decact*theta(i)*cml(i)+decact*bdens(lay)
     &           *fmobil(i)*kf*cref*((cml(i)/cref)**frexp)
        dectot = dectot + ctrans*dt*dz(i)
        
C --- plant uptake and diffusion between mobile and immobile region
        crot = tscf*qrot(i)*cml(i)/dz(i)
        rottot = rottot + tscf*qrot(i)*cml(i)*dt
        if (fmobil(i).lt.0.99d0) then
          cdiff = kmobil*(cml(i)-cil(i))
        else
          cdiff = 0.0
        endif

C --- lateral drainage
        cdrtot = 0.0
        do 100 level = 1,5
          if (qdra(level,i) .gt. 0.0) then
            cdrtot = cdrtot+qdra(level,i)*cml(i)/dz(i)
          else
            cdrtot = cdrtot+qdra(level,i)*cdrain/dz(i)
          endif
 100    continue
C --- cumulative amount of solutes to lateral drainage
        isqdra  = isqdra + cdrtot*dz(i)
        sqdra   = sqdra + cdrtot*dz(i)*dt
        samount = samount + cdrtot*dz(i)

C --- desorption flux of water flowing through cracks
        if ((SWCRACK.eq.1) .and. z(i) .gt. cralev) then
          desflu(i) = deswat * (cml(i) - desconc) * difdes * dz(i)
          destot = destot + desflu(i)
        else
          desflu(i) = 0.0
        endif

C --- conservation equation for the substance
        cmsy(i) = cmsy(i)+(cfluxb-cfluxt-desflu(i))/dz(i)+
     &            (-ctrans-crot-cdrtot-cdiff)*dt +
     &            adsflu(i) * crackc * dt / dz(i)

C --- iteration procedure for calculation of cml
        differ = .true.
        if (cmsy(i).lt.1.d-6) then
          cmsy(i) = 0.0
          cml(i)  = 0.0
        else
          if (abs(frexp-1.0d0).lt.0.001d0) then
            cml(i) = cmsy(i)/(theta(i)+bdens(lay)*kf*fmobil(i))
          else 
            if (cml(i).lt.1.d-6) cml(i) = 1.d-6
 200        if (differ) then
              old    = cml(i)
              dummy  = bdens(lay)*kf*(cml(i)/cref)**(frexp-1.0d0)
              cml(i) = cmsy(i)/(theta(i)+dummy*fmobil(i))
              if (abs(cml(i)-old).lt.rer*cml(i)) differ = .false.
              goto 200
            endif
          endif
        endif

C --- Immobile phase ---------------------------------------------------

        if (fmobil(i).lt.0.99d0) then

C --- transformation of the substance
          ctrans = (decact*(1.0d0-fmobil(i))*(thetim(lay)*cil(i)+
     &             bdens(lay)*kf*cref*(cil(i)/cref)**frexp))
          dectot = dectot + ctrans*dt*dz(i)

C --- conservation equation for the substance
          cisy(i) = cisy(i)+(-ctrans+cdiff)*dt

C --- iteration procedure for calculation of cil
          differ = .true.
          if (cisy(i).lt.1.d-6) then
            cisy(i) = 0.0
            cil(i)  = 0.0
          else
            if (abs(frexp-1.0d0).lt.0.001d0) then
              cil(i) = cisy(i) / (thetim(lay) + bdens(lay)*kf) /
     &                 (1.0d0-fmobil(i))
            else
              if (cil(i).lt.1.d-6) cil(i) = 1.d-6
 300          if (differ) then
                old    = cil(i)
                dummy  = bdens(lay)*kf*(cil(i)/cref)**(frexp-1.0d0)
                cil(i) = cisy(i)/(thetim(lay)+dummy)/(1.0d0-
     &                   fmobil(i))
                if (abs(cil(i)-old).lt.rer*cil(i)) differ = .false.
                goto 300
              endif
            endif
          endif
        endif

C --- mean solute concentration in liquid
        cl(i)  = (theta(i)*cml(i)+thetim(lay)*(1.0d0-fmobil(i))
     &           *cil(i))/(theta(i)+thetim(lay)*(1.0d0-fmobil(i)))
        cfluxt = cfluxb

C --- next compartment...
 500  continue

C --- solute balance in aquifer for breakthrough curve
      if (swbr .eq. 1) then
        if (qdrtot .gt. 0.0) then
          cdrain = cdrain + dt/(poros + bdens(layer(numnod))*kfsat) * 
     &           ( (isqdra - qdrtot*cdrain)/haquif -
     &           decsat*cdrain*(poros + bdens(layer(numnod))*kfsat) )
	  else
          cdrain = cdrain + dt/(poros + bdens(layer(numnod))*kfsat) * 
     &           ( isqdra/haquif -
     &           decsat*cdrain*(poros + bdens(layer(numnod))*kfsat) )
	  endif
	endif

C --- concentration of drainage water if DIVDRA = 1
      if (qdrtot.gt.1.d-6) then
        sdrain = samount/qdrtot
      else
        sdrain = 0.0
      endif

C --- total solute amounts for balance
      sampro = 0.0
      do 520 i = 1,numnod
        sampro = sampro + (cmsy(i) + cisy(i)) * dz(i)
 520  continue
      sampro = sampro + csurf
      samcra = crackw*crackc
      samaq  = cdrain*poros*haquif

C --- determine maximum timestep
      dtsolu = dtmax
      do 550 i = 1,numnod
        thetav = inpola(i+1)*theta(i)+inpolb(i)*theta(i+1)
        diffus = ddif * (thetav**2.33)/(thetsl(lay)*thetsl(lay))
        dispr  = diffus+ldis*abs(q(i))/theta(i)
        if (dispr.lt.1.0d-8) dispr = 1.0d-8
        dummy  = dz(i)*dz(i)*theta(i)/2.0/dispr
        dtsolu = min(dtsolu,dummy)
 550  continue

      dtsolu = max(0.001d0, dtsolu)

      return
      end

C-----------------------------------------------------------------------
      REAL*8 FUNCTION THENODE (NODE,HEAD,LAYER,FLGENU,COFGEN,
     & HTABLE,THETLO,ALFAMG,THETAR,THETAS)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calc. water content at nodal point NODE from
C                          pressure head HEAD
C     Subroutines called : -
C     Functions called   : -                                           
C     File usage         : -
C     Differences SWAP/SWAPS: None
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global 
      INTEGER NODE,LAYER(MACP),THETLO(MAHO)

      REAL*8  HEAD,COFGEN(8,MAHO),HTABLE(MAHO,99)
      REAL*8  ALFAMG(MACP),THETAR(MACP),THETAS(MACP)

      LOGICAL FLGENU(MAHO)

C ----------------------------------------------------------------------
C --- local

      INTEGER lay,posit

      REAL*8  help
C ----------------------------------------------------------------------

C --- soil layer of the node
      lay = layer(node)

      if (head.ge.-1.0d-2) then

C ---   saturated moisture content
        thenode = thetas(node)
      else
        if (flgenu(lay)) then

C ---     soil physical data presented according to Van Genuchten
C ---     first compute |alpha * h| ** n
          help = (abs(alfamg(node)*head))**cofgen(6,lay)

C ---     add 1 and raise to the power m
          help = (1.0d0 + help) ** cofgen(7,lay)

C ---     now compute theta
          thenode = thetar(node)+(thetas(node)-thetar(node))/help    
        else

C ---     soil physical data presented in a table
          if (head.le.htable(lay,thetlo(lay))) then

C ---       h is below lowest value of table
            thenode = (0.01d0 * thetlo(lay))
          else
            posit   = thetlo(lay)
 100        posit   = posit+1
            if (head.gt.htable(lay,posit)) goto 100

C ---       linear interpolation between water contents
            thenode = 0.01d0*((posit-1)+(head-htable(lay,posit-1))/
     $                (htable(lay,posit)-htable(lay,posit-1)))
          endif
        end if
      end if

      return
      end


      SUBROUTINE TRIDAG (N,A,B,C,R,U,ErrorCode)
************************************************************************
** Date      : 15/9/99                                    
** Purpose:    Solves for a vector U a tridiagonal linear set.         *
** References:                                                         *
** Press, W.H., B.P. Flannery, S.A. Teukolsky & W.T. Vetterling, 1989. *
** Numerical Recipes in FORTRAN. Cambridge University Press, New York. *
** pp 40-41                                                            *
************************************************************************
** Input:     N -      Number of equations                             *
**            A,B,C -  Coefficients of the matrix                      *
**            R -      known vector                                    *
** Output:    U -      solved vector                                   *
************************************************************************
      IMPLICIT NONE
      Include 'Param.Fi'

C --- Global declarations
C --- (i.i) Input
      INTEGER	  ErrorCode,N
      REAL*8	  A(N), B(N), C(N), R(N)
C --- Output
      REAL*8    U(N)
C --- Parameters
      REAL*8    Small
      PARAMETER (Small = 0.3d-37)

C --- Local declarations
      INTEGER   I
      REAL*8	  Gamma(MACP), Beta
C ----------------------------------------------------------------------
C --- (1) If B(1)=0 then rewrite the equations as a set of order N-1
C     to eliminate U(2)

      If (Abs(B(1)).LT.Small) Then
        ErrorCode = 100
      else
C ---   (2) Decomposition and forward substitution
        Beta = B(1)
        U(1) = R(1) / Beta
        do I = 2, N
          Gamma(I) = C(I-1) / Beta
          Beta = B(I) - A(I)*Gamma(I)

C ---     (2.1) If Beta=0 then go to another algorithm including
C ---     elimination with pivoting
          If (Abs(Beta).LT.Small) Then
		    ErrorCode = 110
		else
            U(I) = (R(I) - A(I)*U(I-1)) / Beta
		end if
        enddo

	  if (ErrorCode .eq. 0) then
C ---     (3) Back substitution
          Do I = N-1, 1, -1
            U(I) = U(I) - Gamma(I+1)*U(I+1)
          End Do
	  end if
      end if

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE UPDATE (NUMNOD,LAYER,H,HM1,INDEKS,TAU,FLGENU,COFGEN,
     & THETAR,THETAS,ALFAMG,THETLO,THETHI,DMCH,DMCC,DIMOCA,
     & SWMOBI,FM1,PF1,SLOPFM,HTABLE,THETA,NODGWL,DISNOD)
C ----------------------------------------------------------------------
C     Date               : 23/10/2000    
C     Purpose            : Check for hysteretic reversal and update      
C                          model parameters            
C ----------------------------------------------------------------------
      IMPLICIT NONE
      INCLUDE 'PARAM.FI'

C --- global
      INTEGER  NUMNOD,LAYER(MACP),INDEKS(MACP),THETLO(MAHO),THETHI(MAHO)
      INTEGER  SWMOBI,NODGWL

      REAL*8   FM1(MAHO),PF1(MAHO),SLOPFM(MAHO),DISNOD(MACP+1)
      REAL*8   H(MACP),HM1(MACP),TAU,THETAR(MACP),THETAS(MACP)
      REAL*8   THETA(MACP),COFGEN(8,MAHO),ALFAMG(MACP)
      REAL*8   DMCC(MAHO,99),DMCH(MAHO,99),DIMOCA(MACP),HTABLE(MAHO,99)

      LOGICAL  FLGENU(MAHO)
C ----------------------------------------------------------------------

C --- local
      INTEGER node,lay,indtem(macp)
      REAL*8  dmcnode,delp,sew,sed,fvalue,prhnode
C ----------------------------------------------------------------------

C --- check for reversal
      do node = 1,numnod
        delp = hm1(node)-h(node)
        if (delp/float(indeks(node)).gt.tau.and.
     &     h(node).lt.-10.0d0 .and. h(node).gt.-1.0d3) then
          indtem(node) = -indeks(node)
        else
          indtem(node) = indeks(node)
        endif
      end do

C --- change parameters scanning curves
      do 100 node = 1,numnod
        lay = layer(node)

C ---   no change
        if (indtem(node).eq.indeks(node) .or. 
     &     abs(cofgen(4,lay)-cofgen(8,lay)) .lt. 1.d-4) goto 100

C ---   relative saturation
        sew = (1.0d0+(cofgen(8,lay)*(-h(node)))**cofgen(6,lay))
     &        **(-cofgen(7,lay)) 
        sed = (1.0d0+(cofgen(4,lay)*(-h(node)))**cofgen(6,lay))
     &        **(-cofgen(7,lay))

C ---   change index
        indeks(node) = -1*indeks(node)

C ---   update alfa, thetar and thetas
        if (indeks(node).eq.1) then
C ---     wetting branch
          alfamg(node) = cofgen(8,lay)
          thetas(node) = cofgen(2,lay)
          thetar(node) = (theta(node)-thetas(node)*sew)/(1.0d0-sew)

C ---     check on thetar(node) value, if needed correction of h
          fvalue = thetar(node)
          if (thetar(node).lt.cofgen(1,lay)) thetar(node)=cofgen(1,lay)
          if (thetar(node).gt.cofgen(2,lay)) thetar(node)=cofgen(2,lay)
          if (abs(fvalue-thetar(node)) .gt. 1.d-10)
     &       h(node) = prhnode(node,theta(node),layer,disnod,h,flgenu,
     &                 thetar,thetas,cofgen,alfamg,htable)

        else
C ---     drying branch
          alfamg(node) = cofgen(4,lay)
          thetar(node) = cofgen(1,lay)
          thetas(node) = thetar(node)+(theta(node)-thetar(node))/sed

C ---     check on thetas(node) value, if needed correction of h
          fvalue = thetas(node)
          if (thetas(node).lt.cofgen(1,lay)) thetas(node)=cofgen(1,lay)
          if (thetas(node).gt.cofgen(2,lay)) thetas(node)=cofgen(2,lay)
          if (abs(fvalue-thetas(node)) .gt. 1.d-10)
     &       h(node) = prhnode(node,theta(node),layer,disnod,h,flgenu,
     &                 thetar,thetas,cofgen,alfamg,htable)
        endif

C ---   update capacity 
        dimoca(node) = dmcnode(node,h(node),layer,flgenu,cofgen,thetlo,
     &                 thethi,dmch,dmcc,thetas,thetar,alfamg,
     &                 swmobi,fm1,pf1,slopfm,htable,theta)

 100  continue

      return
      end

C ----------------------------------------------------------------------
      subroutine WBALLEV (wls,t,wlstab,wst,sttab,dt,runots,rapdra,
     &                    qdrd,cqdrd,cwsupp,cwout,WLSOLD)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : close surface water balance using 
C                          given (input) surface water levels   

C --- Compare storage at T + drainage fluxes with storage at T+DT
C --- The difference will be either discharge or supply
C --- Update totals (of discharge, supply and qdrd)

C     Subroutines called :                          
C     Functions called   : wstlev                          
C     File usage         :
C     Differences SWAP/SWAPS: None        
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      REAL*8  wls,t,wlstab(2*MAWLS)
      REAL*8  wst,sttab(22,2),dt,runots,rapdra,qdrd,cqdrd,cwsupp
      REAL*8  cwout,AFGEN

C --- local
      REAL*8  wlsold,wstold,wstrest,wdis,wsupp,wstlev

C --- Error handling
      INTEGER GetErrorStatus
	EXTERNAL GetErrorStatus

C ----------------------------------------------------------------------
C --- wlsold gets w-level of previous time step
       wlsold = wls

C --- fetch new level from input series
       wls = AFGEN (WLSTAB,2*MAWLS,T+DT+1.0D0)

C --- determine surface water storage for level(t-dt) and level(t)
      wstold = wstlev(wlsold,sttab) 
      wst    = wstlev(wls,sttab)
	if (GetErrorStatus().ne.0) return 


C --- determine from the surface water storages and the qdrain whether
C --- supply has taken place during period (t)-(t+dt) or water has 
C --- been discharged
      wstrest = wstold + (qdrd + rapdra)*dt + runots - wst

C --- if supply was needed, set discharge to zero
      if (wstrest.le.0.0) then
        wdis  = 0.0
        wsupp = -wstrest/dt

C --- if discharge has taken place, set supply to zero
      else
        wdis  = wstrest/dt
        wsupp = 0.0
      endif

C --- cumulation of water balance terms
      cqdrd  = cqdrd  + qdrd*dt
      cwsupp = cwsupp + wsupp*dt
      cwout  = cwout  + wdis*dt

      return
      end

C ----------------------------------------------------------------------
      SUBROUTINE WLEVBAL (
     & HLEVD,NRPRI,impend,nmper,swman,imper,
     & wlstar,wls,hbweir,gwl,wlsman,gwlcrit,nphase,dropr,sttab,wscap,
     & dt,runots,rapdra,wst,zbotdr,alphaw,betaw,qdrd,cqdrd,cwsupp,
     & cwout,wlsbak,osswlm, T,NUMNOD,THETAS,THETA,DZ,VCRIT,NODHD,HCRIT,
     & H,SWQHR,QQHTAB,HQHTAB,wldip,hwlman,vtair,overfl,numadj,intwl)
C ----------------------------------------------------------------------
C     Date               : 29/9/99
C     Purpose            : calculate surf. water level from water balance

C --- Set target level wlstar:
C --- SWMAN = 1: HBWEIR
C --- SWMAN = 2: from table (4e #2) and within adjustment period and 
C --- taking into account the maximum drop rate. 
C --- Calculate level for maximum supply wlstara (= wlstar-wldip)

C --- Calculate storage at wlstar and at wlstara
C ---   If wlstara is above deepest bottom level then water supply 
C ---   capacity is set to maximum value otherwise both wsttara and
C ---   wsmax are set to zero

C --- Calculate max. new storage: old + incoming/outgoing fluxes
C --- 1 System falls dry: set supply to maximum,
C       set wls at bottom of deepest channel
C --- 2 System will not become full: set supply to maximum
C       calculate new level from storage
C --- 3 System becomes full under maximum supply conditions,
C ---   Determine how, first without supply
C --- 3a wlstara cannot be reached: calculate required supply,
C ---    to reach wlstara and set wls to wlstara
C --- 3b system can be filled above wlstara, however no discharge:
C ---    wls is calculated from storage and is somewhere between
C ---    wlstar and wlstara, no supply, no discharge
C --- 3c wlstar can be reached :wsupp = 0;
C ---    Now check whether there are automatic weirs for keeping the 
C ---    level at target value or that the discharge relationship
C ---    determines new level:
C --- 3c1 SWMAN=2: calculate discharge
C ---     check whether there is enough discharge capacity:
C ---     in case SWQHR = 1: calculate discap
C ---     in case SWQHR = 2: find discap from table
C ---     if sufficient capacity wls = wlstar otherwise set overflow
C --- 3c2 SWMAN=1 or overflow
C ---     Calculate highest possible discharge, if still unsufficient 
C ---     to handle present drain fluxes: stop - system overflow
C ---     otherwise start iteration procedure to determine new level,
C ---     storage and discharge:

C ***********************************************
C --- Iteration Procedure:
C --- Establish lower and upper bounds: hbweir(imper) and +100cm
C --- Calculate storage and discharge for point halfway: wsti, wdisi
C --- Calculate new storage: if higher than wsti then adjust lower
C --- bound to point halfway otherwise adapt upper bound to point halfway
C --- Continue until upper and lower bounds converge (< 0.001 cm)   
C --- Ready: update wls, wst and wdis
C ***********************************************

C     Subroutines called :                          
C     Functions called   : wstlev                          
C     File usage         :
C ----------------------------------------------------------------------
      IMPLICIT NONE
      Include 'Param.Fi'

C --- global
      INTEGER impend(10),nmper,swman(10),nphase(10),NUMNOD
      INTEGER NODHD(10),SWQHR,NRPRI,NUMADJ,intwl(10)

      REAL*8  wlstar,hbweir(10),gwl,wlsman(10,10)
      REAL*8  dropr(10),sttab(22,2),wscap(10),dt,runots,rapdra
      REAL*8  wls,wst,osswlm,gwlcrit(10,10)
      REAL*8  zbotdr(5),alphaw(10),betaw(10),qdrd,cqdrd,cwsupp
      REAL*8  cwout,wlsbak(4),T,DZ(MACP),VCRIT(10,10),HCRIT(10,10)
      REAL*8  QQHTAB(10,10),HQHTAB(10,10),WLDIP(10),HWLMAN
      REAL*8  VTAIR
      REAL*8  THETAS(MACP),THETA(MACP),H(MACP)
      LOGICAL overfl
     
C --- local
      INTEGER imper,iphase,NODE,Iday
      REAL*8  wlstx,wsttar,dvmax,wstmax,wsupp,wdis,wlstarb
      REAL*8  wover,discap,wlsl,wlsu,wlsi,wsti,wdisi,wstn
      REAL*8  wprod1,wprod2,oscil,wlevst,wstlev,wlstara
      REAL*8  wsttara,qhtab,rday,wsmax
      LOGICAL hlevd

C --- Error handling
      character*80 errTxt,errSrc        
      integer GetErrorStatus
	external GetErrorStatus

C --- retain values of local variables
      save
C-----------------------------------------------------------------------

C --- Error handling
      errSrc = 'WLEVBAL' 

C --- resetting of flag for overflowing of automatic weir
      overfl = .false.

C --- memorizing previous target level
      wlstarb = wlstar

C --- determine which management period the model is in:
      imper   = 0
 100  imper   = imper + 1

C --- error handling
      if (imper .gt. nmper) then
	  errTxt = 'error sw-management periods'
	  call SetFatalError(9999,errTxt,errSrc)
	  return
      endif
	
	if (INT(T+1.0D0) .gt. impend(imper)) goto 100

C --- determine the target sw-level:
      if (swman(imper) .eq. 1) then

C --- In the case of a fixed weir the 'target level' is set to the 
C --- weir crest, for later use in calculations to determine whether 
c --- there is any outflow at all (see below):
        wlstar = hbweir(imper)
      else

C ---  For automatic weir, determine the target level of the surface 
C ---  water, dependent on groundwater level:

c ---   only adjust it if new subperiod, with length intwl(imper) has 
c       started  (or if it is is the first call (not possible in SWAPS))
        rday = (T+1.0D0)/intwl(imper)
        iday = int(rday)

        if (abs(rday - 1.0*iday).lt.0.00001d0 .or. .not.hlevd) then

          iphase = nphase(imper)
 200      if (gwl.gt.gwlcrit(imper,iphase).and.iphase.gt.1) then
            iphase = iphase-1
            goto 200
          endif

C --- compare total air volume with VCRIT, adapt iphase
          VTAIR = 0.0
          do NODE = 1,NUMNOD
            VTAIR = VTAIR + (THETAS(NODE)-THETA(NODE))
     &              *abs(DZ(NODE))
          enddo
 214      if (VTAIR.lt.VCRIT(imper,iphase).AND.IPHASE.gt.1) then
            iphase = iphase - 1
            goto 214
          endif

C --- compare H(nodhd(imper)) with HCRIT, adapt iphase 
 218      HWLMAN = H(NODHD(imper))
          if (HWLMAN.gt.HCRIT(imper,iphase).AND.IPHASE.gt.1) then
            iphase = iphase - 1
            goto 218
          endif

          wlstx = wlsman(imper,iphase)
        else

c --- use old level
          wlstx = wlstarb
        endif

C ---   if the level must drop, then do not let it drop at more than 
C ---   the specified rate:
        if (wlstx .lt. wlstar .and. dropr(imper) .gt. 0.001d0) then
          wlstar = wlstar - dropr(imper)*dt
          if (wlstar .lt. wlstx) wlstar = wlstx
        else
          wlstar = wlstx
        endif
      endif

C --- counter of adjustments
      if (abs(wlstar-wlstarb) .gt. 0.00001D0) then
         numadj = numadj + 1
      endif

C --- storage for the 'target level'
      wsttar = wstlev(wlstar,sttab)

C --- level and storage for "max. level for supply"
      wlstara = wlstar - wldip(imper)
      if (wlstara .gt. (zbotdr(1+NRPRI)+1.D-4)) then
         wsttara = wstlev(wlstara,sttab)
         wsmax   = wscap(imper)
      else
         wsttara = 0.0
         wsmax   = 0.0
      endif
      
	if (GetErrorStatus().ne.0) return

C --- determine whether the system will become full (target level or
C --- level of weir crest):
      dvmax   = (qdrd + rapdra + wsmax) * dt + runots
      wstmax  = wst + dvmax

      if (wstmax .lt. 1.0D-7) then
C ---   storage decreases to zero, then the surface water system 
C ---   falls dry; set surface water supply to maximum: 
        if (wstmax .lt. -0.1D0) then

C ---     error handling
		ErrTxt = 'error algorithm for sw falling dry'
          call SetFatalError(9999,errTxt,errSrc)
	    return
        endif
       
	  wsupp  = wsmax
        wdis   = 0.0
        wst    = 0.0
        wls    = zbotdr(NRPRI+1)

      elseif (wstmax .ge. 0.0 .and. wstmax .lt. wsttara) then
C --- system will not become full - set supply to max. capacity
        wsupp  = wsmax
        wdis   = 0.0
        wst    = wstmax

C --- calculate new level from storage
        wls = wlevst(wst,sttab)
	  if (GetErrorStatus().ne.0) return
      else

C --- determine how system will become full: with or without needing
C     surface water supply; try first without any supply:
        dvmax  = (qdrd + rapdra) * dt + runots
        wstmax = wst + dvmax
        if (wstmax .le. wsttara) then

C --- apparently supply is needed for reaching target level, system 
C     is made full up to level wlstara, because supply is controllable:
          wsupp  = (wsttara - wst - (qdrd+rapdra)*dt-runots)/dt
          wdis   = 0.0
          wst    = wsttara
          wls    = wlstara
        elseif (wstmax .le. wsttar) then

C --- apparently drainage is more than sufficient for filling system
C     to a level above the target level FOR SUPPLY, but not enough
C     for generating discharge; so calculate level from storage:
          wsupp = 0.0
          wdis  = 0.0
          wst   = wstmax
          wls   = wlevst(wst,sttab)
	if (GetErrorStatus().ne.0) return
        else

C --- drainage water is more than sufficient for reaching target
C     level;  now see whether there are automatic weirs for keeping
C     the level at target value, or that the discharge relationship
C     determines new level:
          wsupp = 0.
          if (swman(imper) .eq. 2) then

C --- the outflow equals the drainage flux, plus the storage
C     excess (or deficit !) in the target situation compared to
C     the actual situation: 
            wdis = (wst-wsttar + (qdrd+rapdra)*dt + runots )/dt

C --- now check whether the weir has enough discharge capacity 
C     at this water level
            if (SWQHR.eq.1) then
              wover = wls - hbweir(imper)
              discap = alphaw(imper) * (wover**betaw(imper))
            elseif (SWQHR.eq.2) then

C --- interpolate QH table
              discap = qhtab(wlstar,imper,hqhtab,qqhtab)
            endif
            if (discap .gt. wdis) then
              wls    = wlstar
              wst    = wsttar
              overfl = .false.
            else
              overfl = .true.
            endif
          endif
          if (swman(imper) .eq. 1 .or. overfl) then

C --- determine level from q-h relationship, and also account
C     for change in storage (see below)
C     check first that the system does not overflow 
            if (SWQHR.eq.1) then
              wover = sttab(1,1) - hbweir(imper)
              discap = alphaw(imper) * (wover**betaw(imper))
            elseif (SWQHR.eq.2) then
              discap = QQHTAB(imper,1)
            endif
            
C ---         error handling
            if (discap .lt. (qdrd+rapdra+runots/dt)) then
              errTxt = 'surface water system has overflowed!'
              call SetFatalError(9999,errTxt,errSrc)
	        return
            endif

C --- iteration procedure for determining new level, storage, 
C     and discharge
            wlsl = hbweir(imper)
            wlsu = sttab(1,1)

C --- find storage and discharge for intermediate point
 700        wlsi   = (wlsl + wlsu) * 0.5
            wsti  = wstlev(wlsi,sttab)
            if (SWQHR.eq.1) then
              wdisi = alphaw(imper)*(wlsi-hbweir(imper))**betaw(imper)
            else
              wdisi = qhtab(wlsi,imper,hqhtab,qqhtab)
            endif
            wstn  = wst + (qdrd + rapdra - wdisi)*dt + runots
            if (wstn .lt. wsti) then
              wlsu = wlsi 
            else
              wlsl = wlsi
            endif
            if ((wlsu - wlsl) .gt. 0.001d0) then

C --- continue iteration procedure:
              goto 700
            else

C --- updating of sw-parameters:
              wls  = wlsi
              wst  = wstn
              wdis = wdisi
            endif
          endif
        endif
      endif

C --- cumulative terms:
      cqdrd  = cqdrd  + qdrd*dt
      cwsupp = cwsupp + wsupp*dt
      cwout  = cwout  + wdis*dt

C --- updating registration of last four levels, for noting oscillation
      wlsbak(1) = wlsbak(2)
      wlsbak(2) = wlsbak(3)
      wlsbak(3) = wlsbak(4)
      wlsbak(4) = wls
      wprod1 = (wlsbak(2)-wlsbak(1))*(wlsbak(3)-wlsbak(2))
      wprod2 = (wlsbak(3)-wlsbak(2))*(wlsbak(4)-wlsbak(3))
      if (wprod1.lt.0.0 .and. wprod2.lt.0.0) then
        oscil = abs(wlsbak(3)-wlsbak(2)) 
        if (oscil .gt. osswlm) then
          errTxt = 'sw-level oscillation; advise: reduction of dtmax' 
		call SetError(9999,errTxt,errSrc)
	  endif
      endif
 
c --- register that first call for target level has been
      if (.not.hlevd) then
        hlevd = .true.
      end if

      return
      end


C ----------------------------------------------------------------------
      real*8 function WLEVST (wstor,sttab)
C ----------------------------------------------------------------------
C     Date               : 23/08/99
C     Purpose            : calculate surface water level from    
C                          surface water storage, using table
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C     Differences SWAP/SWAPS: None
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER i

      REAL*8  wstor, sttab(22,2),dwst

      character*80 errTxt,errSrc

C ----------------------------------------------------------------------
	errSrc = 'WLEVST'


      if (wstor .lt. sttab(22,2)) then
        errTxt = 'surface water storage below bottom of table'
        call SetFatalError(9999,errTxt,errSrc)
	  return
      endif
      if (wstor .gt. sttab(1,2)) then
        errTxt = 'surface water storage above top of table'
	  call SetFatalError(9999,errTxt,errSrc)
	  return 
      endif
      i = 0
 100  i = i+1
      if (.not.(wstor.ge.sttab(i+1,2).and.
     &          wstor.le.sttab(i,2))) goto 100
      dwst   = (wstor - sttab(i+1,2)) / (sttab(i,2) - sttab(i+1,2))
      wlevst = sttab(i+1,1) + dwst * (sttab(i,1) - sttab(i+1,1))

      return
      end


C ----------------------------------------------------------------------
      real*8 function WSTLEV (wlev,sttab)
C-----------------------------------------------------------------------
C     Date               : 23/08/99
C     Purpose            : calculate surface water storage from  
C                          surface water level
C     Subroutines called : -
C     Functions called   : -
C     File usage         : -
C     Differences SWAP/SWAPS: None
C ----------------------------------------------------------------------
      IMPLICIT NONE

      INTEGER i

      REAL*8  wlev, sttab(22,2),dwl

      character*80 errTxt,errSrc
C ----------------------------------------------------------------------
	errSrc = 'WSTLEV'

      if (wlev .lt. sttab(22,1)) then
	  errTxt = 'surface water level below bottom of table'
        call SetFatalError(9999,errTxt,errSrc)
	  return
	endif
      if (wlev .gt. sttab(1,1)) then
        errTxt = 'surface water level above bottom of table'
        call SetFatalError(9999,errTxt,errSrc)
	  return
	endif

      i = 0
 100  i = i+1
      if (.not.(wlev.ge.sttab(i+1,1).and.wlev.le.sttab(i,1))) goto 100
      dwl    = (wlev - sttab(i+1,1)) / (sttab(i,1) - sttab(i+1,1))
      wstlev = sttab(i+1,2) + dwl * (sttab(i,2) - sttab(i+1,2))

      return
      end



