************************************************************************
** FILE:
**    SWerrlib.FOR - common program errorcode for SWAP and SWAPS programs
**
** FILE IDENTIFICATION:
**    $Id: swerrlib.for 1.6 2002/09/02 08:57:10Z Kroes Exp $
** DESCRIPTION:
C --- Error functions and subroutines

C ----------------------------------------------------------------------
      subroutine ClearErrorLevel(aerrSev)
C ----------------------------------------------------------------------
C --- This subprogram removes all errors of a given level from the erro
C --- module. For removal of all error messages use ErrorInit.
C -I- anErrSev - The level for which errors are removed
C ----------------------------------------------------------------------
      integer aerrSev
C ----------------------------------------------------------------------
      call ErrorClearLevel(aErrSev)

      Return
      End

C ----------------------------------------------------------------------
      subroutine ErrorClearLevel(anErrSev)
C ----------------------------------------------------------------------
C --- Remove all errors with severity level = anErrSev from the list.
C --- The severity level need not be present.
C -I- anErrSev - Severity level to be removed from list
C ----------------------------------------------------------------------
      integer    anErrSev
C ----------------------------------------------------------------------
      integer MaxMessages
      parameter (MaxMessages = 100)

      character*80 ErrTxt(MaxMessages)
      character*80 ErrSrc(MaxMessages)
      common /CMN_ERRAPI_CHR/ ErrTxt, ErrSrc

      integer*4 ErrNr(MaxMessages)
      integer*4 ErrSev(MaxMessages)
      integer*4 NumMessages
      common /CMN_ERRAPI_NUM/ NumMessages, ErrNr, ErrSev
C ----------------------------------------------------------------------
	integer i,j
C ----------------------------------------------------------------------
	i = 1
   10 continue
      if (i.le. NumMessages) then
	  if (ErrSev(i) .eq. anErrSev) then
          do j = i , NumMessages - 1
            ErrSev(j) = ErrSev(j+1)
            ErrNr(j)  = ErrNr(j+1)
            ErrTxt(j) = ErrTxt(j+1)
            ErrSrc(j) = ErrSrc(j+1)
          enddo
          NumMessages = NumMessages - 1         
        endif
        i = i + 1
	  goto 10
      endif       

      Return
      End

C ----------------------------------------------------------------------
	integer function ErrorFindSev(aErrSev)
C ----------------------------------------------------------------------
C --- Return the number of errors in error module with a severity
C --- equal to aErrSev.
C ----------------------------------------------------------------------
      integer aErrSev
C ----------------------------------------------------------------------
      integer MaxMessages
      parameter (MaxMessages = 100)

      character*80 ErrTxt(MaxMessages)
      character*80 ErrSrc(MaxMessages)
      common /CMN_ERRAPI_CHR/ ErrTxt, ErrSrc

      integer*4 ErrNr(MaxMessages)
      integer*4 ErrSev(MaxMessages)
      integer*4 NumMessages
      common /CMN_ERRAPI_NUM/ NumMessages, ErrNr, ErrSev
C --- Local ------------------------------------------------------------
      integer Count, i
C ----------------------------------------------------------------------
      Count = 1
	do i = 1 , NumMessages
	  if (ErrSev(i) .eq. aErrSev) goto 10
	enddo
	Count = 0

   10 continue
      ErrorFindSev = Count

      Return
	End

C ----------------------------------------------------------------------
      subroutine ErrorGetError(aNth,aErrSev,aErrNr,aErrTxt,aErrSrc)
C ----------------------------------------------------------------------
C --- Find the N-th occurence of an error with severity = aErrSev. 
C --- If it is present then return the information of that error. If
C --- it is not present return empty strings and don't change aerrnr.

C -I- integer  aNth    - The number to return
C -I- integer  aErrSev - The severiry to look for
C -O- integer  aErrNr  - Number of error 
C -O- char*(*) aErrTxt - Text associated with error
C -O- char*(*) aErrSrc - Source of error
C ----------------------------------------------------------------------
      integer       aNth,aErrSev,aErrNr
	character*(*) aErrTxt,aErrSrc
C ----------------------------------------------------------------------
      integer MaxMessages
      parameter (MaxMessages = 100)

      character*80 ErrTxt(MaxMessages)
      character*80 ErrSrc(MaxMessages)
      common /CMN_ERRAPI_CHR/ ErrTxt, ErrSrc

      integer*4 ErrNr(MaxMessages)
      integer*4 ErrSev(MaxMessages)
      integer*4 NumMessages
      common /CMN_ERRAPI_NUM/ NumMessages, ErrNr, ErrSev

C --- Local
      integer i, k
C ----------------------------------------------------------------------
      i = 1
	k = 0
   10 continue
      if (i.le.NumMessages .and. k.lt. aNth) then
	  if (aErrSev.eq.ErrSev(i)) k = k + 1
        i = i + 1
        goto 10
      endif

C --- Now the situation if k = aNth and k > 0 then okee else false
C --- (aNth <= 0)

      aErrNr = -1
	aErrSrc = ' '
	aErrTxt = ' '
      if (k.gt.0 .and. aNth.eq.k) then
	  aErrNr = ErrNr(k)
        aErrSrc = ErrSrc(k)
        aErrTxt = ErrTxt(k)
	endif

      Return 
      End

C ----------------------------------------------------------------------
      subroutine ErrorInit
C ----------------------------------------------------------------------
C --- This subprogram initializes the error module
C ----------------------------------------------------------------------
      integer NumMessages
C ----------------------------------------------------------------------
	call ErrorInitError(NumMessages)

      call SetErrorStatus(0)

      Return
      End

C ----------------------------------------------------------------------
      subroutine ErrorInitError(NumMessages)
C ----------------------------------------------------------------------
C --- This subprogram (re) initializes the ERROR API. It just
C --- sets the number of messages to 0.
C ----------------------------------------------------------------------
      integer NumMessages
C ----------------------------------------------------------------------
      NumMessages = 0

      Return
	End

C ----------------------------------------------------------------------
      subroutine ErrorSetError(aErrNr, aErrTxt, aErrSev, aErrSrc)
C ----------------------------------------------------------------------
C --- This subprogram adds an error to the list. If there is not
C --- enough room for the error no notification is given.

C -I- integer  aErrNr  - Identification of error
C -I- char*(*) aErrTxt - Brief text describing the error
C -I- integer  aErrSev - Severity of error
C -I- char*(*) aErrSrc - Error source
C ----------------------------------------------------------------------
	integer       aErrNr
	character*(*) aErrTxt
	integer       aErrSev
	character*(*) aErrSrc
C ----------------------------------------------------------------------
      integer MaxMessages
      parameter (MaxMessages = 100)

      character*80 ErrTxt(MaxMessages)
      character*80 ErrSrc(MaxMessages)
      common /CMN_ERRAPI_CHR/ ErrTxt, ErrSrc

      integer*4 ErrNr(MaxMessages)
      integer*4 ErrSev(MaxMessages)
      integer*4 NumMessages
      common /CMN_ERRAPI_NUM/ NumMessages, ErrNr, ErrSev
C ----------------------------------------------------------------------
      if (aErrSev .gt. 10 .or. aErrSev .lt. 0) then
	  NumMessages = NumMessages + 1
        errNr(NumMessages) = 0
        errTxt(NumMessages) = 'Error level not in range 0..10'
        errSev(NumMessages) = 2
        errSrc(NumMessages) = 'ERROR module'
      else if (NumMessages .lt. MaxMessages) then
	  NumMessages = NumMessages + 1
        errNr(NumMessages) = aErrNr
        errTxt(NumMessages) = aErrTxt
        errSev(NumMessages) = aErrSev
        errSrc(NumMessages) = aErrSrc
      end if

      Return
      End

C ----------------------------------------------------------------------
	Subroutine GetError(Nth,aErrNr,aErrTxt,aErrSrc)
C ----------------------------------------------------------------------
C --- This function returns the N-th Error message. If the Nth 
C --- error message is not found both the textfields are filled
C --- with blanks.

C -I- integer  Nth      - The N-th error message 
C -I- integer  anErrNr  - The error number
C -I- char*(*) anErrTxt - The error text
C -I- char*(*) anErrSrc - The error source
C ----------------------------------------------------------------------
      integer   Nth
	integer   aErrNr
	character*(*) aErrTxt
	character*(*) aErrSrc
C ----------------------------------------------------------------------
      call ErrorGetError(Nth, 2, aErrNr, aErrTxt, aErrSrc)

      Return
      End

C ----------------------------------------------------------------------
	Subroutine GetFatalError(Nth,aErrNr,aErrTxt,aErrSrc)
C ----------------------------------------------------------------------
C --- This function returns the N-th fatal error message. If the Nth 
C --- fatal error message is not found both the textfields are filled
C --- with blanks.

C -I- integer  Nth      - The N-th fatal error message 
C -O- integer  anErrNr  - The error number
C -O- char*(*) anErrTxt - The error text
C -O- char*(*) anErrSrc - The error source
C ----------------------------------------------------------------------
      integer   Nth
	integer   aErrNr
	character*(*) aErrTxt
	character*(*) aErrSrc
C ----------------------------------------------------------------------
      call ErrorGetError(Nth, 3, aErrNr, aErrTxt, aErrSrc)

      Return
      End

C ----------------------------------------------------------------------
	Subroutine GetInformational(Nth,aErrNr,aErrTxt,aErrSrc)
C ----------------------------------------------------------------------
C --- This function returns the N-th informational message. If the Nth 
C --- informational message is not found both the textfields are filled
C --- with blanks.

C -I- integer  Nth      - The N-th informational message 
C -O- integer  anErrNr  - The error number
C -O- char*(*) anErrTxt - The error text
C -O- char*(*) anErrSrc - The error source
C ----------------------------------------------------------------------
      integer   Nth
	integer   aErrNr
	character*(*) aErrTxt
	character*(*) aErrSrc
C ----------------------------------------------------------------------
      call ErrorGetError(Nth, 0, aErrNr, aErrTxt, aErrSrc)

      Return
      End

C ----------------------------------------------------------------------
	integer function GetErrorStatus()
C ----------------------------------------------------------------------
C --- global variables
      integer ErrorStatus
	common /CMD_ErrorStatus/ ErrorStatus
C ----------------------------------------------------------------------
      GetErrorStatus = ErrorStatus

      Return
      End

C ----------------------------------------------------------------------
	integer function GetModelStatus()
C ----------------------------------------------------------------------
C --- global variables
      integer ModelStatus
	common /CMD_ModelStatus/ ModelStatus
C ----------------------------------------------------------------------
      GetModelStatus = ModelStatus

      Return
      End

C ----------------------------------------------------------------------
      Subroutine GetUserError(Nth,aErrSev, aErrNr,aErrTxt,aErrSrc)
C ----------------------------------------------------------------------
C --- This function returns the N-th message with severity = anErrSev. If the
C --- message is not found both the textfields are filled
C --- with blanks.

C -I- integer  Nth      - The N-th message with severity level = anErrSev
C -I- integer  anErrSev - The severity level to look for
C -O- integer  anErrNr  - The error number
C -O- char*(*) anErrTxt - The error text
C -O- char*(*) anErrSrc - The error source
C ----------------------------------------------------------------------
      integer   Nth
      integer   aErrNr
      integer   aErrSev
      character*(*) aErrTxt
      character*(*) aErrSrc
C ----------------------------------------------------------------------
      call ErrorGetError(Nth, aErrSev, aErrNr, aErrTxt, aErrSrc)

      Return
      end

C ----------------------------------------------------------------------
	Subroutine GetWarning(Nth ,aErrNr ,aErrTxt ,aErrSrc)
C ----------------------------------------------------------------------
C --- This function returns the N-th warning message. If the Nth 
C --- warning message is not found both the textfields are filled
C --- with blanks.

C -I- IN:   integer  Nth      - The N-th warning message 
C -O- OUT:  integer  anErrNr  - The error number
C -O- OUT:  char*(*) anErrTxt - The error text
C -O- OUT:  char*(*) anErrSrc - The error source
C ----------------------------------------------------------------------
      integer   Nth
	integer   aErrNr
	character*(*) aErrTxt
	character*(*) aErrSrc
C ----------------------------------------------------------------------
      call ErrorGetError(Nth, 1, aErrNr, aErrTxt, aErrSrc)

      Return
      end

C ----------------------------------------------------------------------
      Logical Function HasErrors()
C ----------------------------------------------------------------------
C --- This functions returns true if any errors messages are present
C --- in the error module. If there are no errors messages 
C --- the function returns false.
C ----------------------------------------------------------------------
C --- Called functions
	integer ErrorFindSev
C ----------------------------------------------------------------------
	HasErrors = ErrorFindSev(2) .gt. 0

      Return
      End

C ----------------------------------------------------------------------
      Logical Function HasFatalErrors()
C ----------------------------------------------------------------------
C --- This functions returns true if any fatal error messages are present
C --- in the error module. If there are no fatal error messages 
C --- the function returns false.
C ----------------------------------------------------------------------
C --- Called functions
	integer ErrorFindSev
C ----------------------------------------------------------------------
	HasFatalErrors = ErrorFindSev(3) .gt. 0

      Return
      End

C ----------------------------------------------------------------------
      Logical Function HasInformationals()
C ----------------------------------------------------------------------
C --- This functions returns true if any informational messages are present
C --- in the error module. If there are no informational messages 
C --- the function returns false.
C ----------------------------------------------------------------------
C --- Called functions
	integer ErrorFindSev
C ----------------------------------------------------------------------
	HasInformationals = ErrorFindSev(0) .gt. 0

      Return
      End

C ----------------------------------------------------------------------
      Logical Function HasUserErrors(anErrSev)
C ----------------------------------------------------------------------
C --- See if errors with non predefined levels have occurred

C -I- integer  anErrSev - Severity level for which is searched
C ----------------------------------------------------------------------
	integer anErrSev

C --- Called functions
	integer ErrorFindSev
C ----------------------------------------------------------------------
	HasUserErrors = ErrorFindSev(anErrSev) .gt. 0

      Return
      End

C ----------------------------------------------------------------------
      Logical Function HasWarnings()
C ----------------------------------------------------------------------
C --- This functions returns true if any warning messages are present
C --- in the error module. If there are no warning messages 
C --- the function returns false.
C ----------------------------------------------------------------------
C --- Called functions
	integer ErrorFindSev
C ----------------------------------------------------------------------
	HasWarnings = ErrorFindSev(1) .gt. 0

      Return
      End      

C ----------------------------------------------------------------------
      Subroutine SetError(errNr, errTxt, errSrc)
C ----------------------------------------------------------------------
C --- This subprogram sets an error message. If the error module
C --- cannot store the message no error is given.

C -I- integer  anErrNr  - an error number
C -I- char*(*) anErrTxt - a brief error text
C -I- char*(*) anErrSrc - the source that supplied the error message
C ----------------------------------------------------------------------
	integer errNr
	character*(*) errTxt
	character*(*) errSrc
C ----------------------------------------------------------------------
      call ErrorSetError(errNr, errTxt, 2, ErrSrc)

      Return
      End

C ----------------------------------------------------------------------
      Subroutine SetFatalError(errNr, errTxt, errSrc)
C ----------------------------------------------------------------------
C --- This subprogram sets a fatal error message. If the error module
C --- cannot store the message no error is given.

C -I- integer  anErrNr  - an error number
C -I- char*(*) anErrTxt - a brief error text
C -I- char*(*) anErrSrc - the source that supplied the error message
C ----------------------------------------------------------------------
	integer errNr
	character*(*) errTxt
	character*(*) errSrc
C ----------------------------------------------------------------------
      call ErrorSetError(errNr, errTxt, 3, ErrSrc)

	call SetErrorStatus(100)

      Return
      End

C ----------------------------------------------------------------------
	Subroutine SetInformational(errNr, errTxt, errSrc)
C ----------------------------------------------------------------------
C --- This subprogram sets an informational message. If the error module
C --- cannot store the message no error is given.

C -I- integer  anErrNr  - an error number
C -I- char*(*) anErrTxt - a brief error text
C -I- char*(*) anErrSrc - the source that supplied the error message
C ----------------------------------------------------------------------
	integer errNr
	character*(*) errTxt
	character*(*) errSrc
C ----------------------------------------------------------------------
      call ErrorSetError(errNr, errTxt, 0, ErrSrc)

      Return
      End

C ----------------------------------------------------------------------
      subroutine SetErrorStatus(i)
C ----------------------------------------------------------------------
	integer i

C ---global variables
      integer ErrorStatus
	common /CMD_ErrorStatus/ ModelError
C ----------------------------------------------------------------------
      if (ErrorStatus .ne. 0) then
	  if (i .ge. 100) ModelError = ModelError + i
	else
	  ErrorStatus = i
	endif

      Return
      End


C ----------------------------------------------------------------------
      subroutine SetModelStatus(i)
C ----------------------------------------------------------------------
	integer i

C ---global variables
      integer ModelStatus
	common /CMD_ModelStatus/ ModelStatus
C ----------------------------------------------------------------------
      if (ModelStatus .ne. 0) then
	  if (i .ge. 100) ModelStatus = ModelStatus + i
	else
	  ModelStatus = i
	endif

      Return
      End

C ----------------------------------------------------------------------
      Subroutine SetUserError(errNr, errTxt, errSev, errSrc)
C ----------------------------------------------------------------------
C --- This subprogram sets an user error message. If the error module
C --- cannot store the message no error is given. The severity level
C --- must be between 4 and 10. If it is not the error is ignored.

C -I- integer  anErrNr  - an error number
C -I- char*(*) anErrTxt - a brief error text
C -I- integer  anErrSev - an error severity level
C -I- char*(*) anErrSrc - the source that supplied the error message
C ----------------------------------------------------------------------
	integer errNr
	integer errSev
	character*(*) errTxt
	character*(*) errSrc
C ----------------------------------------------------------------------
      call ErrorSetError(errNr, errTxt, ErrSev, ErrSrc)

      Return
      End

C ----------------------------------------------------------------------
      Subroutine SetWarning(errNr, errTxt, errSrc)
C ----------------------------------------------------------------------
C --- This subprogram sets a warning message. If the error module
C --- cannot store the message no error is given.

C -I- integer  anErrNr  - an error number
C -I- char*(*) anErrTxt - a brief error text
C -I- char*(*) anErrSrc - the source that supplied the error message
C ----------------------------------------------------------------------
	integer errNr
	character*(*) errTxt
	character*(*) errSrc
C ----------------------------------------------------------------------
      call ErrorSetError(errNr, errTxt, 1, ErrSrc)

      Return
      End

C ----------------------------------------------------------------------
	Subroutine WriteErrors(ierr)
C ----------------------------------------------------------------------
C --- Purpose: generate list of errors
C ----------------------------------------------------------------------
	Implicit none

	integer MaxMessages
	parameter (MaxMessages = 100)

	character*80 ErrTxt(MaxMessages)
	character*80 ErrSrc(MaxMessages)
	common/CMN_ERRAPI_CHR/ErrTxt,ErrSrc

	integer*4 ErrNr(MaxMessages)
	integer*4 ErrSev(MaxMessages)
	integer*4 NumMessages
	common/CMN_ERRAPI_NUM/ NumMessages,ErrNr,ErrSev

C --- Local
	integer ee,ierr
C ---------------------------------------------------------------------
C --- if errors open err.log and write messages
      if (NumMessages.gt.0) then
	  write (ierr,1001)
	  do ee = 1,NumMessages
           write(ierr,1000)errNr(ee),ErrSrc(ee),ErrTxt(ee),ErrSev(ee)
        end do
	endif
1001  format('code source',7x,'summary',76x,'severity')
1000  format(i4,1x,a12,1x,a80,i4)	   

	return
	end
